@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -11,9 +11,7 @@
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
@@ -22,22 +20,16 @@ log = logging.getLogger(__name__)
 
 # Standard library imports
 import codecs
-import errno
 import os
 import shutil
 import sys
 import tempfile
 
-# External imports
-
-# Bokeh imports
-
 #-----------------------------------------------------------------------------
 # Globals and constants
 #-----------------------------------------------------------------------------
 
 __all__ = (
-    'makedirs_ok_if_exists',
     'TmpDir',
     'with_directory_contents',
     'with_temporary_file',
@@ -48,21 +40,7 @@ __all__ = (
 # General API
 #-----------------------------------------------------------------------------
 
-def makedirs_ok_if_exists(path):
-    '''
-
-    '''
-    try:
-        os.makedirs(path)
-    except IOError as e:  # pragma: no cover (py3 only)
-        if e.errno != errno.EEXIST:
-            raise e
-    except OSError as e:  # pragma: no cover (py2 only)
-        if e.errno != errno.EEXIST:
-            raise e
-    return path
-
-class TmpDir(object):
+class TmpDir:
     '''
 
     '''
@@ -92,10 +70,9 @@ def with_directory_contents(contents, func):
         for filename, file_content in contents.items():
             path = os.path.join(dirname, filename)
             if file_content is None:
-                # make a directory
-                makedirs_ok_if_exists(path)
+                os.makedirs(path, exist_ok=True)
             else:
-                makedirs_ok_if_exists(os.path.dirname(path))
+                os.makedirs(os.path.dirname(path), exist_ok=True)
                 with codecs.open(path, 'w', 'utf-8') as f:
                     f.write(file_content)
         return func(os.path.realpath(dirname))
@@ -114,13 +91,27 @@ def with_file_contents(contents, func, dir=None, suffix=''):
 
     with_temporary_file(with_file_object, dir=dir, suffix=suffix)
 
+async def with_file_contents_async(contents, func, dir=None, suffix=''):
+    '''
+
+    '''
+    async def with_file_object(f):
+        f.write(contents.encode("UTF-8"))
+        f.flush()
+        # Windows will get mad if we try to rename it without closing,
+        # and some users of with_file_contents want to rename it.
+        f.close()
+        await func(f.name)
+
+    await with_temporary_file_async(with_file_object, dir=dir, suffix=suffix)
+
 def with_temporary_file(func, dir=None, suffix=''):
     '''
 
     '''
     if dir is None:
         dir = _LOCAL_TMP
-    import tempfile
+
     # Windows throws a permission denied if we use delete=True for
     # auto-delete, and then try to open the file again ourselves
     # with f.name. So we manually delete in the finally block
@@ -132,7 +123,25 @@ def with_temporary_file(func, dir=None, suffix=''):
         f.close()
         os.remove(f.name)
 
-class WorkingDir(object):
+async def with_temporary_file_async(func, dir=None, suffix=''):
+    '''
+
+    '''
+    if dir is None:
+        dir = _LOCAL_TMP
+
+    # Windows throws a permission denied if we use delete=True for
+    # auto-delete, and then try to open the file again ourselves
+    # with f.name. So we manually delete in the finally block
+    # below.
+    f = tempfile.NamedTemporaryFile(dir=dir, delete=False, suffix=suffix)
+    try:
+        await func(f)
+    finally:
+        f.close()
+        os.remove(f.name)
+
+class WorkingDir:
     '''
 
     '''
@@ -160,4 +169,4 @@ class WorkingDir(object):
 #-----------------------------------------------------------------------------
 
 _LOCAL_TMP = os.path.abspath("./build/tmp")
-makedirs_ok_if_exists(_LOCAL_TMP)
+os.makedirs(_LOCAL_TMP, exist_ok=True)
