@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -17,20 +17,15 @@ method of any ``Handler`` objects that it is configured with.
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
 # Imports
 #-----------------------------------------------------------------------------
 
-# Standard library imports
-
-# External imports
-
 # Bokeh imports
+from .application import Application
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -44,8 +39,6 @@ __all__ = (
 # General API
 #-----------------------------------------------------------------------------
 
-from .application import Application
-
 #-----------------------------------------------------------------------------
 # Dev API
 #-----------------------------------------------------------------------------
