@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -24,9 +24,7 @@ A client session has two primary uses:
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
@@ -34,16 +32,14 @@ log = logging.getLogger(__name__)
 #-----------------------------------------------------------------------------
 
 # Standard library imports
-
-# External imports
-from six.moves.urllib.parse import quote_plus
+from urllib.parse import quote_plus
 
 # Bokeh imports
 from ..document import Document
-from ..resources import _SessionCoordinates, DEFAULT_SERVER_HTTP_URL
+from ..resources import DEFAULT_SERVER_HTTP_URL, _SessionCoordinates
 from ..util.browser import NEW_PARAM
-from ..util.session_id import generate_session_id
-from ..util.string import format_docstring
+from ..util.token import generate_jwt_token, generate_session_id
+from .states import ErrorReason
 from .util import server_url_for_websocket_url, websocket_url_for_server_url
 
 #-----------------------------------------------------------------------------
@@ -213,7 +209,7 @@ def show_session(session_id=None, url='default', session=None, browser=None, new
         controller.open(server_url + "?bokeh-session-id=" + quote_plus(session_id),
                         new=NEW_PARAM[new])
 
-class ClientSession(object):
+class ClientSession:
     ''' Represents a websocket connection to a server-side session.
 
     Each server session stores a Document, which is kept in sync with the
@@ -295,6 +291,22 @@ class ClientSession(object):
         ''' Whether this session is currently connected. '''
         return self._connection.connected
 
+    @property
+    def error_reason(self):
+        return self._connection.error_reason
+
+    @property
+    def error_code(self):
+        return self._connection.error_code
+
+    @property
+    def error_detail(self):
+        return self._connection.error_detail
+
+    @property
+    def url(self):
+        return self._connection.url
+
     @property
     def document(self):
         ''' A :class:`~bokeh.document.Document` that will be kept in sync with
@@ -311,6 +323,11 @@ class ClientSession(object):
         ''' A unique ID for this session. '''
         return self._id
 
+    @property
+    def token(self):
+        ''' A JWT token to authenticate the session. '''
+        return generate_jwt_token(self.id)
+
     # Public methods ----------------------------------------------------------
 
     def connect(self):
@@ -334,24 +351,22 @@ class ClientSession(object):
         '''
         self._connection.force_roundtrip()
 
-    def loop_until_closed(self, suppress_warning=False):
-        ''' Execute a blocking loop that runs and executes event callbacks
-        until the connection is closed (e.g. by hitting Ctrl-C).
+    def check_connection_errors(self):
+        ''' Raises an error, when the connection could not have been
+        established.
 
-        While this method can be used to run Bokeh application code "outside"
-        the Bokeh server, this practice is HIGHLY DISCOURAGED for any real
-        use case. This function is intended to facilitate testing ONLY.
+        Should be used, after a call to connect.
+
+        Returns:
+            None
 
         '''
-
-        suppress_warning # shut up flake
-
-        from bokeh.util.deprecation import deprecated
-        deprecated("ClientSession.loop_until_closed is deprecated, and will be removed in an eventual 2.0 release. "
-                   "Run Bokeh applications directly on a Bokeh server instead. See:\n\n"
-                   "    https//docs.bokeh.org/en/latest/docs/user_guide/server.html\n")
-
-        self._connection.loop_until_closed()
+        if not self.connected:
+            if self.error_reason is ErrorReason.HTTP_ERROR:
+                if self.error_code == 404:
+                    raise OSError(f"Check your application path! The given Path is not valid: {self.url}")
+                raise OSError(f"We received an HTTP-Error. Disconnected with error code: {self.error_id}, given message: {self.error_message}")
+            raise OSError("We failed to connect to the server (to start the server, try the 'bokeh serve' command)")
 
     def pull(self):
         ''' Pull the server's state and set it as session.document.
@@ -363,8 +378,7 @@ class ClientSession(object):
 
         '''
         self.connect()
-        if not self.connected:
-            raise IOError("Cannot pull session document because we failed to connect to the server (to start the server, try the 'bokeh serve' command)")
+        self.check_connection_errors()
 
         if self.document is None:
             doc = Document()
@@ -401,8 +415,7 @@ class ClientSession(object):
                 raise ValueError("Cannot push() a different document from existing session.document")
 
         self.connect()
-        if not self.connected:
-            raise IOError("Cannot push session document because we failed to connect to the server (to start the server, try the 'bokeh serve' command)")
+        self.check_connection_errors()
         self._connection.push_doc(doc)
         if self._document is None:
             self._attach_document(doc)
@@ -468,6 +481,15 @@ class ClientSession(object):
     def _handle_patch(self, message):
         message.apply_to_document(self.document, self)
 
+    def _loop_until_closed(self):
+        ''' Execute a blocking loop that runs and executes event callbacks
+        until the connection is closed (e.g. by hitting Ctrl-C).
+
+        This function is intended to facilitate testing ONLY.
+
+        '''
+        self._connection.loop_until_closed()
+
     def _notify_disconnected(self):
         ''' Called by the ClientConnection we are using to notify us of disconnect.
 
@@ -493,5 +515,3 @@ class ClientSession(object):
 #-----------------------------------------------------------------------------
 # Code
 #-----------------------------------------------------------------------------
-
-__doc__ = format_docstring(__doc__)
