@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -12,20 +12,18 @@ define common named colors.
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
 # Imports
 #-----------------------------------------------------------------------------
 
-# Standard library imports
-
-# External imports
-
 # Bokeh imports
+from . import groups, named
+from .color import Color
+from .hsl import HSL
+from .rgb import RGB
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -43,12 +41,7 @@ __all__ = (
 # General API
 #-----------------------------------------------------------------------------
 
-from .color import Color
-from .hsl import HSL
-from .rgb import RGB
 
-from . import groups
-from . import named
 
 #-----------------------------------------------------------------------------
 # Dev API
