@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -11,9 +11,6 @@ The following subcommands are available:
 build
     Manage and build a bokeh extension
 
-html
-    Create standalone HTML files for one or more applications
-
 info
     Print information about Bokeh and Bokeh server configuration
 
@@ -23,9 +20,6 @@ init
 json
     Create JSON files for one or more applications
 
-png
-    Create standalone PNG files for one or more applications
-
 sampledata
     Download the bokeh sample data sets
 
@@ -38,7 +32,4 @@ serve
 static
     Serve bokehjs' static assets (JavaScript, CSS, images, fonts, etc.)
 
-svg
-    Create standalone SVG files for one or more applications
-
 '''
