@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -42,9 +42,7 @@ This will produce output like what is shown below
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
@@ -53,8 +51,8 @@ log = logging.getLogger(__name__)
 
 # Standard library imports
 import sys
-
-# External imports
+from argparse import Namespace
+from typing import Any, Optional
 
 # Bokeh imports
 from bokeh import __version__
@@ -62,6 +60,7 @@ from bokeh.settings import settings
 from bokeh.util.compiler import nodejs_version, npmjs_version
 from bokeh.util.dependencies import import_optional
 
+# Bokeh imports
 from ..subcommand import Subcommand
 
 #-----------------------------------------------------------------------------
@@ -76,10 +75,20 @@ __all__ = (
 # Private API
 #-----------------------------------------------------------------------------
 
-def _version(modname, attr):
+def if_installed(version_or_none: Optional[str]) -> str:
+    ''' helper method to optionally return module version number or not installed
+
+    :param version_or_none:
+    :return:
+    '''
+    return version_or_none or "(not installed)"
+
+def _version(modname: str, attr: str) -> Optional[Any]:
     mod = import_optional(modname)
     if mod:
         return getattr(mod, attr)
+    else:  # explicit None return for mypy typing
+        return None
 
 #-----------------------------------------------------------------------------
 # General API
@@ -104,14 +113,13 @@ class Info(Subcommand):
 
     )
 
-    def invoke(self, args):
+    def invoke(self, args: Namespace) -> None:
         '''
 
         '''
         if args.static:
             print(settings.bokehjsdir())
         else:
-            if_installed = lambda version_or_none: version_or_none or "(not installed)"
 
             print("Python version      :  %s" % sys.version.split('\n')[0])
             print("IPython version     :  %s" % if_installed(_version('IPython', '__version__')))
