@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -10,9 +10,7 @@
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
@@ -25,13 +23,18 @@ import errno
 import os
 import sys
 import warnings
-
-# External imports
+from typing import Dict, Iterator, List, Optional, Sequence
 
 # Bokeh imports
 from bokeh.application import Application
-from bokeh.application.handlers import ScriptHandler, DirectoryHandler, NotebookHandler
-from bokeh.models.plots import Plot
+from bokeh.application.handlers import (
+    DirectoryHandler,
+    Handler,
+    NotebookHandler,
+    ScriptHandler,
+)
+from bokeh.document import Document
+from bokeh.models import Plot
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -49,7 +52,7 @@ __all__ = (
 # General API
 #-----------------------------------------------------------------------------
 
-def die(message, status=1):
+def die(message: str, status: Optional[int] = 1) -> None:
     ''' Print an error message and exit.
 
     This function will call ``sys.exit`` with the given ``status`` and the
@@ -74,7 +77,7 @@ call "bokeh serve" on the directory instead. For example:
 If this is not the case, renaming main.py will suppress this warning.
 """
 
-def build_single_handler_application(path, argv=None):
+def build_single_handler_application(path: str, argv: Optional[Sequence[str]] = None) -> Application:
     ''' Return a Bokeh application built using a single handler for a script,
     notebook, or directory.
 
@@ -113,6 +116,7 @@ def build_single_handler_application(path, argv=None):
     '''
     argv = argv or []
     path = os.path.abspath(path)
+    handler: Handler
 
     # There are certainly race conditions here if the file/directory is deleted
     # in between the isdir/isfile tests and subsequent code. But it would be a
@@ -138,7 +142,7 @@ def build_single_handler_application(path, argv=None):
 
     return application
 
-def build_single_handler_applications(paths, argvs=None):
+def build_single_handler_applications(paths: List[str], argvs: Optional[Dict[str, List[str]]] = None) -> Dict[str, Application]:
     ''' Return a dictionary mapping routes to Bokeh applications built using
     single handlers, for specified files or directories.
 
@@ -147,7 +151,7 @@ def build_single_handler_applications(paths, argvs=None):
     to generate the mapping.
 
     Args:
-        path (seq[str]) : paths to files or directories for creating Bokeh
+        paths (seq[str]) : paths to files or directories for creating Bokeh
             applications.
 
         argvs (dict[str, list[str]], optional) : mapping of paths to command
@@ -160,7 +164,7 @@ def build_single_handler_applications(paths, argvs=None):
         RuntimeError
 
     '''
-    applications = {}
+    applications: Dict[str, Application] = {}
     argvs = argvs or {}
 
     for path in paths:
@@ -178,7 +182,7 @@ def build_single_handler_applications(paths, argvs=None):
 
 
 @contextlib.contextmanager
-def report_server_init_errors(address=None, port=None, **kwargs):
+def report_server_init_errors(address: Optional[str] = None, port: Optional[int] = None, **kwargs: str) -> Iterator[None]:
     ''' A context manager to help print more informative error messages when a
     ``Server`` cannot be started due to a network problem.
 
@@ -201,7 +205,7 @@ def report_server_init_errors(address=None, port=None, **kwargs):
     '''
     try:
         yield
-    except EnvironmentError as e:
+    except OSError as e:
         if e.errno == errno.EADDRINUSE:
             log.critical("Cannot start Bokeh server, port %s is already in use", port)
         elif e.errno == errno.EADDRNOTAVAIL:
@@ -211,13 +215,16 @@ def report_server_init_errors(address=None, port=None, **kwargs):
             log.critical("Cannot start Bokeh server [%s]: %r", codename, e)
         sys.exit(1)
 
-def set_single_plot_width_height(doc, width, height):
+def set_single_plot_width_height(doc: Document, width: Optional[int], height: Optional[int]) -> None:
     if width is not None or height is not None:
         layout = doc.roots
         if len(layout) != 1 or not isinstance(layout[0], Plot):
             warnings.warn("Width/height arguments will be ignored for this muliple layout. (Size valus only apply when exporting single plots.)")
         else:
             plot = layout[0]
+            # TODO - below fails mypy check
+            # unsure how to handle with typing. width is int base type and class property getter is typing.Int
+            # plot.plot_width  = width if width is not None else plot.plot_width  # doesnt solve problem
             plot.plot_height = height or plot.plot_height
             plot.plot_width  = width or plot.plot_width
 
