@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -11,9 +11,7 @@
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
@@ -23,9 +21,6 @@ log = logging.getLogger(__name__)
 # Standard library imports
 import numbers
 
-# External imports
-from six import string_types
-
 # Bokeh imports
 from .bases import PrimitiveProperty
 
@@ -237,7 +232,7 @@ class String(PrimitiveProperty):
             >>> m.prop = [1, 2, 3]  # ValueError !!
 
     '''
-    _underlying_type = string_types
+    _underlying_type = (str,)
 
 #-----------------------------------------------------------------------------
 # Dev API
