@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -14,6 +14,8 @@ of properties, use the mix-in classes in this module:
 
 * |FillProps| --- properties for fill color and alpha
 
+* |HatchProps| --- properties for hatching pattern, color, alpha, etc.
+
 * |LineProps| --- properties for line color, dashing, width, etc.
 
 * |TextProps| --- properties for text color, font, etc.
@@ -41,6 +43,7 @@ different usage, for more information see the docs for |Include|.
 .. |Include| replace:: :class:`~bokeh.core.properties.Include`
 
 .. |FillProps| replace:: :class:`~bokeh.core.property_mixins.FillProps`
+.. |HatchProps| replace:: :class:`~bokeh.core.property_mixins.HatchProps`
 .. |LineProps| replace:: :class:`~bokeh.core.property_mixins.LineProps`
 .. |TextProps| replace:: :class:`~bokeh.core.property_mixins.TextProps`
 
@@ -48,23 +51,43 @@ different usage, for more information see the docs for |Include|.
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
 # Imports
 #-----------------------------------------------------------------------------
 
-# Standard library imports
-
-# External imports
-
 # Bokeh imports
-from .enums import LineJoin, LineCap, FontStyle, TextAlign, TextBaseline
+from .enums import (
+    FontStyle,
+    HatchPattern,
+    HatchPatternAbbreviation,
+    LineCap,
+    LineJoin,
+    TextAlign,
+    TextBaseline,
+)
 from .has_props import HasProps
-from .properties import Color, ColorSpec, DashPattern, Enum, FontSize, FontSizeSpec, Include, Int, Float, NumberSpec, Percent, String, value
+from .properties import (
+    Color,
+    ColorSpec,
+    DashPattern,
+    Dict,
+    Enum,
+    Float,
+    FontSize,
+    FontSizeSpec,
+    HatchPatternSpec,
+    Include,
+    Instance,
+    Int,
+    NumberSpec,
+    Percent,
+    Size,
+    String,
+    value,
+)
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -72,9 +95,11 @@ from .properties import Color, ColorSpec, DashPattern, Enum, FontSize, FontSizeS
 
 __all__ = (
     'FillProps',
+    'HatchProps',
     'LineProps',
     'TextProps',
     'ScalarFillProps',
+    'ScalarHatchProps',
     'ScalarLineProps',
     'ScalarTextProps',
 )
@@ -93,7 +118,7 @@ Acceptable values are:
 - a 3-tuple of integers (r,g,b) between 0 and 255
 - a 4-tuple of (r,g,b,a) where r,g,b are integers between 0..255 and a is between 0..1
 
-.. _CSS colors: http://www.w3schools.com/cssref/css_colornames.asp
+.. _CSS colors: https://www.w3schools.com/colors/colors_names.asp
 
 """
 
@@ -224,9 +249,55 @@ class ScalarFillProps(HasProps):
 
     '''
 
-    fill_color = Color(default="gray", help=_color_help)
+    fill_color = Color(default="gray", help=_color_help  % "fill paths")
     fill_alpha = Percent(default=1.0, help=_alpha_help)
 
+_hatch_scale_help = """
+A rough measure of the 'size' of the hatching pattern. Generally speaking, the
+higher the number, the more spread out the pattern will be.
+"""
+
+_hatch_pattern_help = """
+Built-in patterns are can either be specified as long names:
+
+%s
+
+or as one-letter abbreviations:
+
+%s
+""" % (", ". join(HatchPattern), ", ". join(repr(x) for x in HatchPatternAbbreviation))
+
+_hatch_weight_help = """
+A width value for line-strokes used in hatching.
+"""
+
+class HatchProps(HasProps):
+    ''' Properties relevant to rendering fill regions.
+
+    Mirrors the BokehJS ``properties.Hatch`` class.
+
+    '''
+
+    hatch_color = ColorSpec(default="black", help=_color_help % "hatching")
+    hatch_alpha = NumberSpec(default=1.0, accept_datetime=False, accept_timedelta=False, help=_alpha_help % "hatching")
+    hatch_scale = NumberSpec(default=12.0, accept_datetime=False, accept_timedelta=False, help=_hatch_scale_help)
+    hatch_pattern = HatchPatternSpec(default=None, help=_hatch_pattern_help)
+    hatch_weight = NumberSpec(default=1.0, accept_datetime=False, accept_timedelta=False, help=_hatch_weight_help)
+    hatch_extra = Dict(String, Instance("bokeh.models.textures.Texture"))
+
+class ScalarHatchProps(HasProps):
+    ''' Properties relevant to rendering fill regions.
+
+    Mirrors the BokehJS ``properties.Hatch`` class.
+
+    '''
+
+    hatch_color = Color(default="black", help=_color_help % "hatching")
+    hatch_alpha = Percent(default=1.0, help=_alpha_help % "hatching")
+    hatch_scale = Size(default=12.0, help=_hatch_scale_help)
+    hatch_pattern = String(default=None, help=_hatch_pattern_help)  # String to accommodate user custom values
+    hatch_weight = Size(default=1.0, help=_hatch_weight_help)
+    hatch_extra = Dict(String, Instance("bokeh.models.textures.Texture"))
 
 _line_width_help = """
 Stroke width in units of pixels.
@@ -271,7 +342,7 @@ class TextProps(HasProps):
     '''
     base_text_props = Include(_BaseTextProps, use_prefix=False)
 
-    text_font_size = FontSizeSpec(value("12pt"))
+    text_font_size = FontSizeSpec(value("16px"))
 
     text_color = ColorSpec(default="#444444", help=_color_help % "fill text")
 
@@ -291,7 +362,7 @@ class ScalarTextProps(HasProps):
     base_text_props = Include(_BaseTextProps, use_prefix=False)
 
     # XXX not great
-    text_font_size = FontSize("12pt")
+    text_font_size = FontSize("16px")
 
     text_color = Color(default="#444444", help=_color_help % "fill text")
 
