@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -11,9 +11,7 @@
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
@@ -21,26 +19,26 @@ log = logging.getLogger(__name__)
 #-----------------------------------------------------------------------------
 
 # Standard library imports
-
-# External imports
-
-# Bokeh imports
+import contextlib
+from typing import Set
 
 #-----------------------------------------------------------------------------
 # Globals and constants
 #-----------------------------------------------------------------------------
-__silencers__ = set()
+
+__silencers__: Set[int] = set()
 
 __all__ = (
     'check_integrity',
-    'silence'
+    'silence',
+    'silenced',
 )
 
 #-----------------------------------------------------------------------------
 # General API
 #-----------------------------------------------------------------------------
 
-def silence(warning, silence=True):
+def silence(warning: int, silence: bool = True) -> Set[int]:
     ''' Silence a particular warning on all Bokeh models.
 
     Args:
@@ -78,6 +76,13 @@ def silence(warning, silence=True):
         __silencers__.remove(warning)
     return __silencers__
 
+@contextlib.contextmanager
+def silenced(warning: int) -> None:
+    silence(warning, True)
+    try:
+        yield
+    finally:
+        silence(warning, False)
 
 def check_integrity(models):
     ''' Apply validation and integrity checks to a collection of Bokeh models.
