@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -11,9 +11,7 @@
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
@@ -21,15 +19,13 @@ log = logging.getLogger(__name__)
 #-----------------------------------------------------------------------------
 
 # Standard library imports
-from six.moves.urllib.parse import urlparse, quote_plus
-
-# External imports
+from urllib.parse import quote_plus, urlparse
 
 # Bokeh imports
-from ..core.templates import AUTOLOAD_TAG, FILE
+from ..core.templates import AUTOLOAD_REQUEST_TAG, FILE
 from ..resources import DEFAULT_SERVER_HTTP_URL
 from ..util.serialization import make_id
-from ..util.string import encode_utf8, format_docstring
+from ..util.string import format_docstring
 from .bundle import bundle_for_objs_and_resources
 from .elements import html_page_for_render_items
 from .util import RenderItem
@@ -48,7 +44,7 @@ __all__ = (
 # General API
 #-----------------------------------------------------------------------------
 
-def server_document(url="default", relative_urls=False, resources="default", arguments=None):
+def server_document(url="default", relative_urls=False, resources="default", arguments=None, headers=None):
     ''' Return a script tag that embeds content from a Bokeh server.
 
     Bokeh apps embedded using these methods will NOT set the browser window title.
@@ -85,6 +81,10 @@ def server_document(url="default", relative_urls=False, resources="default", arg
             A dictionary of key/values to be passed as HTTP request arguments
             to Bokeh application code (default: None)
 
+       headers (dict[str, str], optional) :
+            A dictionary of key/values to be passed as HTTP Headers
+            to Bokeh application code (default: None)
+
     Returns:
         A ``<script>`` tag that will embed content from a Bokeh Server.
 
@@ -101,15 +101,18 @@ def server_document(url="default", relative_urls=False, resources="default", arg
     src_path += _process_resources(resources)
     src_path += _process_arguments(arguments)
 
-    tag = AUTOLOAD_TAG.render(
+    headers = headers or {}
+
+    tag = AUTOLOAD_REQUEST_TAG.render(
         src_path  = src_path,
         app_path  = app_path,
         elementid = elementid,
+        headers   = headers,
     )
 
-    return encode_utf8(tag)
+    return tag
 
-def server_session(model=None, session_id=None, url="default", relative_urls=False, resources="default"):
+def server_session(model=None, session_id=None, url="default", relative_urls=False, resources="default", headers={}):
     ''' Return a script tag that embeds content from a specific existing session on
     a Bokeh server.
 
@@ -158,6 +161,10 @@ def server_session(model=None, session_id=None, url="default", relative_urls=Fal
             files you'll load separately are of the same version as that of the
             server's, otherwise the rendering may not work correctly.
 
+       headers (dict[str, str], optional) :
+            A dictionary of key/values to be passed as HTTP Headers
+            to Bokeh application code (default: None)
+
     Returns:
         A ``<script>`` tag that will embed content from a Bokeh Server.
 
@@ -181,17 +188,20 @@ def server_session(model=None, session_id=None, url="default", relative_urls=Fal
 
     src_path += _process_app_path(app_path)
     src_path += _process_relative_urls(relative_urls, url)
-    src_path += _process_session_id(session_id)
     src_path += _process_resources(resources)
 
-    tag = AUTOLOAD_TAG.render(
+    headers = dict(headers) if headers else {}
+    headers['Bokeh-Session-Id'] = session_id
+
+    tag = AUTOLOAD_REQUEST_TAG.render(
         src_path  = src_path,
         app_path  = app_path,
         elementid = elementid,
         modelid   = modelid,
+        headers   = headers,
     )
 
-    return encode_utf8(tag)
+    return tag
 
 #-----------------------------------------------------------------------------
 # Dev API
@@ -216,7 +226,7 @@ def server_html_page_for_session(session, resources, title, template=FILE, templ
 
     '''
     render_item = RenderItem(
-        sessionid = session.id,
+        token = session.token,
         roots = session.document.roots,
         use_for_title = True,
     )
@@ -333,19 +343,6 @@ def _process_resources(resources):
         return "&resources=none"
     return ""
 
-def _process_session_id(session_id):
-    ''' Return a session ID HTML argument to add to a Bokeh server URL
-
-    Args:
-        session_id (str) :
-            The session id to use
-
-    Returns:
-        str
-
-    '''
-    return "&bokeh-session-id=" + session_id
-
 def _src_path(url, elementid):
     ''' Return a base autoload URL for a given element ID
 
