@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -11,9 +11,7 @@
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
@@ -22,11 +20,9 @@ log = logging.getLogger(__name__)
 
 # Standard library imports
 from collections import OrderedDict
-from bokeh.util.future import collections_abc # goes away with py2
+from collections.abc import Sequence
 from contextlib import contextmanager
 
-# External imports
-
 # Bokeh imports
 from ..document.document import Document
 from ..model import Model, collect_models
@@ -56,7 +52,7 @@ __all__ = (
 # Dev API
 #-----------------------------------------------------------------------------
 
-class FromCurdoc(object):
+class FromCurdoc:
     ''' This class merely provides a non-None default value for ``theme``
     arguments, since ``None`` itself is a meaningful value for users to pass.
 
@@ -87,7 +83,7 @@ def OutputDocumentFor(objs, apply_theme=None, always_new=False):
     * If passed a subset of Document.roots, then OutputDocumentFor temporarily "re-homes"
       the models in a new bare Document that is only available inside the context manager.
 
-    * If passed a list of models that have differnet documents, then OutputDocumentFor
+    * If passed a list of models that have different documents, then OutputDocumentFor
       temporarily "re-homes" the models in a new bare Document that is only available
       inside the context manager.
 
@@ -118,13 +114,13 @@ def OutputDocumentFor(objs, apply_theme=None, always_new=False):
     # Note: Comms handling relies on the fact that the new_doc returned
     # has models with the same IDs as they were started with
 
-    if not isinstance(objs, collections_abc.Sequence) or len(objs) == 0 or not all(isinstance(x, Model) for x in objs):
+    if not isinstance(objs, Sequence) or len(objs) == 0 or not all(isinstance(x, Model) for x in objs):
         raise ValueError("OutputDocumentFor expects a sequence of Models")
 
     def finish(): pass
 
-    docs = set(x.document for x in objs)
-    if None in docs: docs.remove(None)
+    docs = {x.document for x in objs}
+    docs.discard(None)
 
     if always_new:
         def finish(): # NOQA
@@ -148,7 +144,7 @@ def OutputDocumentFor(objs, apply_theme=None, always_new=False):
                 doc = _create_temp_doc(objs)
 
             # we are using all the roots of a single doc, just use doc as-is
-            pass
+            pass  # lgtm [py/unnecessary-pass]
 
         # models have mixed docs, just make a quick clone
         else:
@@ -167,10 +163,10 @@ def OutputDocumentFor(objs, apply_theme=None, always_new=False):
 
     finish()
 
-class RenderItem(object):
 
-    def __init__(self, docid=None, sessionid=None, elementid=None, roots=None, use_for_title=None):
-        if (docid is None and sessionid is None) or (docid is not None and sessionid is not None):
+class RenderItem:
+    def __init__(self, docid=None, token=None, elementid=None, roots=None, use_for_title=None):
+        if (docid is None and token is None) or (docid is not None and token is not None):
             raise ValueError("either docid or sessionid must be provided")
 
         if roots is None:
@@ -179,7 +175,7 @@ class RenderItem(object):
             roots = OrderedDict([ (root, make_globally_unique_id()) for root in roots ])
 
         self.docid = docid
-        self.sessionid = sessionid
+        self.token = token
         self.elementid = elementid
         self.roots = RenderRoots(roots)
         self.use_for_title = use_for_title
@@ -190,13 +186,14 @@ class RenderItem(object):
         if self.docid is not None:
             json["docid"] = self.docid
         else:
-            json["sessionid"] = self.sessionid
+            json["token"] = self.token
 
         if self.elementid is not None:
             json["elementid"] = self.elementid
 
         if self.roots:
             json["roots"] = self.roots.to_json()
+            json["root_ids"] = [root.id for root in self.roots]
 
         if self.use_for_title is not None:
             json["use_for_title"] = self.use_for_title
@@ -209,8 +206,8 @@ class RenderItem(object):
         else:
             return self.to_json() == other.to_json()
 
-class RenderRoot(object):
 
+class RenderRoot:
     def __init__(self, elementid, id, name=None, tags=None):
         self.elementid = elementid
         self.id = id
@@ -223,8 +220,8 @@ class RenderRoot(object):
         else:
             return self.elementid == other.elementid
 
-class RenderRoots(object):
 
+class RenderRoots:
     def __init__(self, roots):
         self._roots = roots
 
@@ -263,7 +260,7 @@ def standalone_docs_json_and_render_items(models, suppress_callback_warning=Fals
     if isinstance(models, (Model, Document)):
         models = [models]
 
-    if not (isinstance(models, collections_abc.Sequence) and all(isinstance(x, (Model, Document)) for x in models)):
+    if not (isinstance(models, Sequence) and all(isinstance(x, (Model, Document)) for x in models)):
         raise ValueError("Expected a Model, Document, or Sequence of Models or Documents")
 
     if submodel_has_python_callbacks(models) and not suppress_callback_warning:
