@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -11,9 +11,7 @@
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
@@ -21,20 +19,21 @@ log = logging.getLogger(__name__)
 #-----------------------------------------------------------------------------
 
 # Standard library imports
-import os
 import io
+import os
 import warnings
 from os.path import abspath
 from tempfile import mkstemp
+from typing import TYPE_CHECKING, Any, List, Optional, Tuple, Union, cast
 
 # External imports
-from six import raise_from, b
+from PIL import Image
 
 # Bokeh imports
+from ..document import Document
 from ..embed import file_html
-from ..resources import INLINE
-from ..util.dependencies import import_required
-from ..util.string import decode_utf8
+from ..models.layouts import LayoutDOM
+from ..resources import INLINE, Resources
 from .util import default_filename
 
 #-----------------------------------------------------------------------------
@@ -42,21 +41,23 @@ from .util import default_filename
 #-----------------------------------------------------------------------------
 
 __all__ = (
-    'create_webdriver',
     'export_png',
+    'export_svg',
     'export_svgs',
     'get_layout_html',
     'get_screenshot_as_png',
     'get_svgs',
-    'terminate_webdriver',
-    'webdriver_control',
 )
 
+if TYPE_CHECKING:
+    from selenium.webdriver.remote.webdriver import WebDriver
+
 #-----------------------------------------------------------------------------
 # General API
 #-----------------------------------------------------------------------------
 
-def export_png(obj, filename=None, height=None, width=None, webdriver=None, timeout=5):
+def export_png(obj: Union[LayoutDOM, Document], *, filename: Optional[str] = None, width: Optional[int] = None,
+        height: Optional[int] = None, webdriver: "Optional[WebDriver]" = None, timeout: int = 5) -> str:
     ''' Export the ``LayoutDOM`` object or document as a PNG.
 
     If the filename is not given, it is derived from the script name (e.g.
@@ -69,12 +70,12 @@ def export_png(obj, filename=None, height=None, width=None, webdriver=None, time
         filename (str, optional) : filename to save document under (default: None)
             If None, infer from the filename.
 
-        height (int) : the desired height of the exported layout obj only if
-            it's a Plot instance. Otherwise the height kwarg is ignored.
-
         width (int) : the desired width of the exported layout obj only if
             it's a Plot instance. Otherwise the width kwarg is ignored.
 
+        height (int) : the desired height of the exported layout obj only if
+            it's a Plot instance. Otherwise the height kwarg is ignored.
+
         webdriver (selenium.webdriver) : a selenium webdriver instance to use
             to export the image.
 
@@ -94,7 +95,7 @@ def export_png(obj, filename=None, height=None, width=None, webdriver=None, time
 
     '''
 
-    image = get_screenshot_as_png(obj, height=height, width=width, driver=webdriver, timeout=timeout)
+    image = get_screenshot_as_png(obj, width=width, height=height, driver=webdriver, timeout=timeout)
 
     if filename is None:
         filename = default_filename("png")
@@ -106,7 +107,44 @@ def export_png(obj, filename=None, height=None, width=None, webdriver=None, time
 
     return abspath(filename)
 
-def export_svgs(obj, filename=None, height=None, width=None, webdriver=None, timeout=5):
+def export_svg(obj: Union[LayoutDOM, Document], *, filename: Optional[str] = None, width: Optional[int] = None,
+        height: Optional[int] = None, webdriver: "Optional[WebDriver]" = None, timeout: int = 5) -> List[str]:
+    ''' Export a layout as SVG file or a document as a set of SVG files.
+
+    If the filename is not given, it is derived from the script name
+    (e.g. ``/foo/myplot.py`` will create ``/foo/myplot.svg``)
+
+    Args:
+        obj (LayoutDOM object) : a Layout (Row/Column), Plot or Widget object to display
+
+        filename (str, optional) : filename to save document under (default: None)
+            If None, infer from the filename.
+
+        width (int) : the desired width of the exported layout obj only if
+            it's a Plot instance. Otherwise the width kwarg is ignored.
+
+        height (int) : the desired height of the exported layout obj only if
+            it's a Plot instance. Otherwise the height kwarg is ignored.
+
+        webdriver (selenium.webdriver) : a selenium webdriver instance to use
+            to export the image.
+
+        timeout (int) : the maximum amount of time (in seconds) to wait for
+            Bokeh to initialize (default: 5)
+
+    Returns:
+        filenames (list(str)) : the list of filenames where the SVGs files are saved.
+
+    .. warning::
+        Responsive sizing_modes may generate layouts with unexpected size and
+        aspect ratios. It is recommended to use the default ``fixed`` sizing mode.
+
+    '''
+    svgs = get_svg(obj, width=width, height=height, driver=webdriver, timeout=timeout)
+    return _write_collection(svgs, filename, "svg")
+
+def export_svgs(obj: Union[LayoutDOM, Document], *, filename: Optional[str] = None, width: Optional[int] = None,
+        height: Optional[int] = None, webdriver: "Optional[WebDriver]" = None, timeout: int = 5) -> List[str]:
     ''' Export the SVG-enabled plots within a layout. Each plot will result
     in a distinct SVG file.
 
@@ -119,12 +157,12 @@ def export_svgs(obj, filename=None, height=None, width=None, webdriver=None, tim
         filename (str, optional) : filename to save document under (default: None)
             If None, infer from the filename.
 
-        height (int) : the desired height of the exported layout obj only if
-            it's a Plot instance. Otherwise the height kwarg is ignored.
-
         width (int) : the desired width of the exported layout obj only if
             it's a Plot instance. Otherwise the width kwarg is ignored.
 
+        height (int) : the desired height of the exported layout obj only if
+            it's a Plot instance. Otherwise the height kwarg is ignored.
+
         webdriver (selenium.webdriver) : a selenium webdriver instance to use
             to export the image.
 
@@ -132,58 +170,27 @@ def export_svgs(obj, filename=None, height=None, width=None, webdriver=None, tim
             Bokeh to initialize (default: 5) (Added in 1.1.1).
 
     Returns:
-        filenames (list(str)) : the list of filenames where the SVGs files are
-        saved.
+        filenames (list(str)) : the list of filenames where the SVGs files are saved.
 
     .. warning::
         Responsive sizing_modes may generate layouts with unexpected size and
         aspect ratios. It is recommended to use the default ``fixed`` sizing mode.
 
     '''
-    svgs = get_svgs(obj, height=height, width=width, driver=webdriver, timeout=timeout)
+    svgs = get_svgs(obj, width=width, height=height, driver=webdriver, timeout=timeout)
 
     if len(svgs) == 0:
         log.warning("No SVG Plots were found.")
-        return
+        return []
 
-    if filename is None:
-        filename = default_filename("svg")
-
-    filenames = []
-
-    for i, svg in enumerate(svgs):
-        if i == 0:
-            filename = filename
-        else:
-            idx = filename.find(".svg")
-            filename = filename[:idx] + "_{}".format(i) + filename[idx:]
-
-        with io.open(filename, mode="w", encoding="utf-8") as f:
-            f.write(svg)
-
-        filenames.append(filename)
-
-    return filenames
+    return _write_collection(svgs, filename, "svg")
 
 #-----------------------------------------------------------------------------
 # Dev API
 #-----------------------------------------------------------------------------
 
-# this is part of the API for this module
-from .webdriver import webdriver_control
-from .webdriver import terminate_webdriver # for back compat
-
-def create_webdriver():
-    ''' Create a new webdriver.
-
-    .. note ::
-        Here for compatibility. Prefer methods on the webdriver_control
-        object.
-
-    '''
-    return webdriver_control.create()
-
-def get_screenshot_as_png(obj, driver=None, timeout=5, **kwargs):
+def get_screenshot_as_png(obj: Union[LayoutDOM, Document], *, driver: "Optional[WebDriver]" = None, timeout: int = 5,
+        resources: Resources = INLINE, width: Optional[int] = None, height: Optional[int] = None) -> Image:
     ''' Get a screenshot of a ``LayoutDOM`` object.
 
     Args:
@@ -198,109 +205,134 @@ def get_screenshot_as_png(obj, driver=None, timeout=5, **kwargs):
             the layout to be rendered.
 
     Returns:
-        cropped_image (PIL.Image.Image) : a pillow image loaded from PNG.
+        image (PIL.Image.Image) : a pillow image loaded from PNG.
 
     .. warning::
         Responsive sizing_modes may generate layouts with unexpected size and
         aspect ratios. It is recommended to use the default ``fixed`` sizing mode.
 
     '''
-    Image = import_required('PIL.Image',
-                            'To use bokeh.io.export_png you need pillow ' +
-                            '("conda install pillow" or "pip install pillow")')
+    from .webdriver import webdriver_control
 
     with _tmp_html() as tmp:
-        html = get_layout_html(obj, **kwargs)
-        with io.open(tmp.path, mode="w", encoding="utf-8") as file:
-            file.write(decode_utf8(html))
+        html = get_layout_html(obj, resources=resources, width=width, height=height)
+        with open(tmp.path, mode="w", encoding="utf-8") as file:
+            file.write(html)
 
         web_driver = driver if driver is not None else webdriver_control.get()
-
-        web_driver.get("file:///" + tmp.path)
         web_driver.maximize_window()
-
-        ## resize for PhantomJS compat
-        web_driver.execute_script("document.body.style.width = '100%';")
-
+        web_driver.get("file:///" + tmp.path)
         wait_until_render_complete(web_driver, timeout)
-
+        [width, height, dpr] = _maximize_viewport(web_driver)
         png = web_driver.get_screenshot_as_png()
 
-        b_rect = web_driver.execute_script(_BOUNDING_RECT_SCRIPT)
+    return (Image.open(io.BytesIO(png))
+                 .convert("RGBA")
+                 .crop((0, 0, width*dpr, height*dpr))
+                 .resize((width, height)))
 
-    image = Image.open(io.BytesIO(png))
-    cropped_image = _crop_image(image, **b_rect)
+def get_svg(obj: Union[LayoutDOM, Document], *, driver: "Optional[WebDriver]" = None, timeout: int = 5,
+        resources: Resources = INLINE, width: Optional[int] = None, height: Optional[int] = None) -> List[str]:
+    from .webdriver import webdriver_control
 
-    return cropped_image
-
-def get_svgs(obj, driver=None, timeout=5, **kwargs):
-    '''
-
-    '''
     with _tmp_html() as tmp:
-        html = get_layout_html(obj, **kwargs)
-        with io.open(tmp.path, mode="wb") as file:
-            file.write(b(html))
+        html = get_layout_html(obj, resources=resources, width=width, height=height)
+        with open(tmp.path, mode="w", encoding="utf-8") as file:
+            file.write(html)
 
         web_driver = driver if driver is not None else webdriver_control.get()
-
         web_driver.get("file:///" + tmp.path)
-
         wait_until_render_complete(web_driver, timeout)
-
-        svgs = web_driver.execute_script(_SVG_SCRIPT)
+        svgs = cast(List[str], web_driver.execute_script(_SVG_SCRIPT))
 
     return svgs
 
-def get_layout_html(obj, resources=INLINE, **kwargs):
+def get_svgs(obj: Union[LayoutDOM, Document], *, driver: "Optional[WebDriver]" = None, timeout: int = 5,
+        resources: Resources = INLINE, width: Optional[int] = None, height: Optional[int] = None) -> List[str]:
+    from .webdriver import webdriver_control
+
+    with _tmp_html() as tmp:
+        html = get_layout_html(obj, resources=resources, width=width, height=height)
+        with open(tmp.path, mode="w", encoding="utf-8") as file:
+            file.write(html)
+
+        web_driver = driver if driver is not None else webdriver_control.get()
+        web_driver.get("file:///" + tmp.path)
+        wait_until_render_complete(web_driver, timeout)
+        svgs = cast(List[str], web_driver.execute_script(_SVGS_SCRIPT))
+
+    return svgs
+
+def get_layout_html(obj: Union[LayoutDOM, Document], *, resources: Resources = INLINE,
+        width: Optional[int] = None, height: Optional[int] = None) -> str:
     '''
 
     '''
     resize = False
-    if kwargs.get('height') is not None or kwargs.get('width') is not None:
+    if width is not None or height is not None:
         # Defer this import, it is expensive
         from ..models.plots import Plot
         if not isinstance(obj, Plot):
             warnings.warn("Export method called with height or width kwargs on a non-Plot layout. The size values will be ignored.")
         else:
             resize = True
-            old_height = obj.plot_height
+
             old_width = obj.plot_width
-            obj.plot_height = kwargs.get('height', old_height)
-            obj.plot_width = kwargs.get('width', old_width)
+            old_height = obj.plot_height
+
+            if width is not None:
+                obj.plot_width = width
+            if height is not None:
+                obj.plot_height = height
+
+    template = r"""\
+    {% block preamble %}
+    <style>
+        html, body {
+            box-sizing: border-box;
+            width: 100%;
+            height: 100%;
+            margin: 0;
+            border: 0;
+            padding: 0;
+            overflow: hidden;
+        }
+    </style>
+    {% endblock %}
+    """
 
     try:
-        html = file_html(obj, resources, title="", suppress_callback_warning=True, _always_new=True)
+        html = file_html(obj, resources, title="", template=template, suppress_callback_warning=True, _always_new=True)
     finally:
         if resize:
-            obj.plot_height = old_height
+            assert isinstance(obj, Plot)
             obj.plot_width = old_width
+            obj.plot_height = old_height
 
     return html
 
-def wait_until_render_complete(driver, timeout):
+def wait_until_render_complete(driver: "WebDriver", timeout: int) -> None:
     '''
 
     '''
     from selenium.webdriver.support.ui import WebDriverWait
     from selenium.common.exceptions import TimeoutException
-    from selenium.webdriver import Firefox
 
-    def is_bokeh_loaded(driver):
-        return driver.execute_script('''
-            const b = window.Bokeh;
-            return b && b.documents && b.documents.length > 0;
-        ''')
+    def is_bokeh_loaded(driver: "WebDriver") -> bool:
+        return cast(bool, driver.execute_script('''
+            return typeof Bokeh !== "undefined" && Bokeh.documents != null && Bokeh.documents.length != 0
+        '''))
 
     try:
         WebDriverWait(driver, timeout, poll_frequency=0.1).until(is_bokeh_loaded)
     except TimeoutException as e:
-        raise_from(RuntimeError('Bokeh was not loaded in time. Something may have gone wrong.'), e)
+        _log_console(driver)
+        raise RuntimeError('Bokeh was not loaded in time. Something may have gone wrong.') from e
 
     driver.execute_script(_WAIT_SCRIPT)
 
-    def is_bokeh_render_complete(driver):
-        return driver.execute_script('return window._bokeh_render_complete;')
+    def is_bokeh_render_complete(driver: "WebDriver") -> bool:
+        return cast(bool, driver.execute_script('return window._bokeh_render_complete;'))
 
     try:
         WebDriverWait(driver, timeout, poll_frequency=0.1).until(is_bokeh_render_complete)
@@ -309,31 +341,93 @@ def wait_until_render_complete(driver, timeout):
                     "a 'bokeh:idle' event to signify that the layout has rendered. "
                     "Something may have gone wrong.")
     finally:
-        # Firefox webdriver does not currently support logs
-        if not isinstance(driver, Firefox):
-            browser_logs = driver.get_log('browser')
-            messages = [ l.get("message") for l in browser_logs if l.get('level') in ['WARNING', 'ERROR', 'SEVERE'] ]
-            if len(messages) > 0:
-                log.warning("There were browser warnings and/or errors that may have affected your export")
-                for message in messages:
-                    log.warning(message)
+        _log_console(driver)
 
 #-----------------------------------------------------------------------------
 # Private API
 #-----------------------------------------------------------------------------
 
-_BOUNDING_RECT_SCRIPT = """
-return document.getElementsByClassName('bk-root')[0].children[0].getBoundingClientRect()
+def _write_collection(items: List[str], filename: Union[str, None], ext: str) -> List[str]:
+    if filename is None:
+        filename = default_filename(ext)
+
+    filenames = []
+
+    for i, item in enumerate(items):
+        if i > 0:
+            idx = filename.find(f".{ext}")
+            filename = filename[:idx] + f"_{i}" + filename[idx:]
+
+        with open(filename, mode="w", encoding="utf-8") as f:
+            f.write(item)
+
+        filenames.append(filename)
+
+    return filenames
+
+def _log_console(driver: "WebDriver") -> None:
+    levels = {'WARNING', 'ERROR', 'SEVERE'}
+    try:
+        logs = driver.get_log('browser')
+    except Exception:
+        return
+    messages = [ log.get("message") for log in logs if log.get('level') in levels ]
+    if len(messages) > 0:
+        log.warning("There were browser warnings and/or errors that may have affected your export")
+        for message in messages:
+            log.warning(message)
+
+def _maximize_viewport(web_driver: "WebDriver") -> Tuple[int, int, int]:
+    calculate_viewport_size = """\
+        const root = document.getElementsByClassName("bk-root")[0]
+        const {width, height} = root.children[0].getBoundingClientRect()
+        return [width, height, window.devicePixelRatio]
+    """
+    viewport_size: Tuple[int, int, int] = web_driver.execute_script(calculate_viewport_size)
+    calculate_window_size = """\
+        const [width, height, dpr] = arguments
+        return [
+            // XXX: outer{Width,Height} can be 0 in headless mode under certain window managers
+            Math.max(0, window.outerWidth - window.innerWidth) + width*dpr,
+            Math.max(0, window.outerHeight - window.innerHeight) + height*dpr,
+        ]
+    """
+    [width, height] = web_driver.execute_script(calculate_window_size, *viewport_size)
+    eps = 100 # XXX: can't set window size exactly in certain window managers, crop it to size later
+    web_driver.set_window_size(width + eps, height + eps)
+    return viewport_size
+
+_SVGS_SCRIPT = """
+const {LayoutDOMView} = Bokeh.require("models/layouts/layout_dom")
+const {PlotView} = Bokeh.require("models/plots/plot")
+
+function* collect_svgs(views) {
+  for (const view of views) {
+    if (view instanceof LayoutDOMView) {
+      yield* collect_svgs(view.child_views.values())
+    }
+    if (view instanceof PlotView && view.model.output_backend == "svg") {
+      const {ctx} = view.canvas_view.compose()
+      yield ctx.get_serialized_svg(true)
+    }
+  }
+}
+
+const root_views = Object.values(Bokeh.index)
+return [...collect_svgs(root_views)]
 """
 
-_SVG_SCRIPT = """
-var serialized_svgs = [];
-var svgs = document.getElementsByClassName('bk-root')[0].getElementsByTagName("svg");
-for (var i = 0; i < svgs.length; i++) {
-    var source = (new XMLSerializer()).serializeToString(svgs[i]);
-    serialized_svgs.push(source);
-};
-return serialized_svgs
+_SVG_SCRIPT = """\
+function* export_svgs(views) {
+  for (const view of views) {
+    // TODO: use to_blob() API in future
+    const {ctx} = view.export("svg")
+    yield ctx.get_serialized_svg(true)
+  }
+}
+
+const root_views = Object.values(Bokeh.index)
+return [...export_svgs(root_views)]
 """
 
 _WAIT_SCRIPT = """
@@ -343,7 +437,7 @@ function done() {
   window._bokeh_render_complete = true;
 }
 
-var doc = window.Bokeh.documents[0];
+var doc = Bokeh.documents[0];
 
 if (doc.is_idle)
   done();
@@ -351,49 +445,42 @@ else
   doc.idle.connect(done);
 """
 
-def _crop_image(image, left=0, top=0, right=0, bottom=0, **kwargs):
-    ''' Crop the border from the layout
 
-    '''
-    return image.crop((left, top, right, bottom))
+class _TempFile:
+    _closed: bool = False
 
-class _TempFile(object):
+    fd: int
+    path: str
 
-    _closed = False
-
-    def __init__(self, prefix="tmp", suffix=""):
+    def __init__(self, *, prefix: str = "tmp", suffix: str = "") -> None:
         self.fd, self.path = mkstemp(prefix=prefix, suffix=suffix)
 
-    def __enter__(self):
+    def __enter__(self) -> "_TempFile":
         return self
 
-    def __exit__(self, exc, value, tb):
+    def __exit__(self, exc: Any, value: Any, tb: Any) -> None:
         self.close()
 
-    def __del__(self):
+    def __del__(self) -> None:
         self.close()
 
-    def close(self):
+    def close(self) -> None:
         if self._closed:
             return
 
         try:
             os.close(self.fd)
-        except (OSError, IOError):
+        except OSError:
             pass
-        finally:
-            self.fd = None
 
         try:
             os.unlink(self.path)
-        except (OSError, IOError):
+        except OSError:
             pass
-        finally:
-            self.path = None
 
         self._closed = True
 
-def _tmp_html():
+def _tmp_html() -> _TempFile:
     return _TempFile(prefix="bokeh", suffix=".html")
 
 #-----------------------------------------------------------------------------
