@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -34,9 +34,7 @@ ensures their proper configuration in many common usage scenarios.
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
@@ -46,9 +44,6 @@ log = logging.getLogger(__name__)
 # Standard library imports
 import os
 
-# External imports
-from six import string_types
-
 # Bokeh imports
 from ..document import Document
 from ..resources import Resources
@@ -66,7 +61,7 @@ __all__ = (
 # General API
 #-----------------------------------------------------------------------------
 
-class State(object):
+class State:
     ''' Manage state related to controlling Bokeh output.
 
     '''
@@ -126,7 +121,7 @@ class State(object):
         defined by external notebook hooks that have been installed.
 
         '''
-        if notebook_type is None or not isinstance(notebook_type, string_types):
+        if notebook_type is None or not isinstance(notebook_type, str):
             raise ValueError("Notebook type must be a string")
         self._notebook_type = notebook_type.lower()
 
