@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -11,9 +11,7 @@
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
@@ -24,14 +22,11 @@ log = logging.getLogger(__name__)
 import math
 from collections import namedtuple
 
-# External imports
-from six import string_types
-
 # Bokeh imports
 from .core.enums import Location
-from .models.tools import ProxyToolbar, ToolbarBox
+from .models.layouts import Box, Column, GridBox, LayoutDOM, Row, Spacer, WidgetBox
 from .models.plots import Plot
-from .models.layouts import LayoutDOM, Box, Row, Column, GridBox, Spacer, WidgetBox
+from .models.tools import ProxyToolbar, ToolbarBox
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -83,18 +78,11 @@ def row(*args, **kwargs):
     sizing_mode = kwargs.pop('sizing_mode', None)
     children = kwargs.pop('children', None)
 
-    children = _handle_children(*args, children=children)
+    children = _parse_children_arg(*args, children=children)
 
-    row_children = []
-    for item in children:
-        if isinstance(item, LayoutDOM):
-            if sizing_mode is not None and _has_auto_sizing(item):
-                item.sizing_mode = sizing_mode
-            row_children.append(item)
-        else:
-            raise ValueError("""Only LayoutDOM items can be inserted into a row. Tried to insert: %s of type %s""" % (item, type(item)))
+    _handle_child_sizing(children, sizing_mode, widget="row")
 
-    return Row(children=row_children, sizing_mode=sizing_mode, **kwargs)
+    return Row(children=children, sizing_mode=sizing_mode, **kwargs)
 
 
 def column(*args, **kwargs):
@@ -128,18 +116,11 @@ def column(*args, **kwargs):
     sizing_mode = kwargs.pop('sizing_mode', None)
     children = kwargs.pop('children', None)
 
-    children = _handle_children(*args, children=children)
+    children = _parse_children_arg(*args, children=children)
 
-    col_children = []
-    for item in children:
-        if isinstance(item, LayoutDOM):
-            if sizing_mode is not None and _has_auto_sizing(item):
-                item.sizing_mode = sizing_mode
-            col_children.append(item)
-        else:
-            raise ValueError("""Only LayoutDOM items can be inserted into a column. Tried to insert: %s of type %s""" % (item, type(item)))
+    _handle_child_sizing(children, sizing_mode, widget="column")
 
-    return Column(children=col_children, sizing_mode=sizing_mode, **kwargs)
+    return Column(children=children, sizing_mode=sizing_mode, **kwargs)
 
 
 def widgetbox(*args, **kwargs):
@@ -166,17 +147,11 @@ def widgetbox(*args, **kwargs):
     sizing_mode = kwargs.pop('sizing_mode', None)
     children = kwargs.pop('children', None)
 
-    children = _handle_children(*args, children=children)
+    children = _parse_children_arg(*args, children=children)
 
-    col_children = []
-    for item in children:
-        if isinstance(item, LayoutDOM):
-            if sizing_mode is not None and _has_auto_sizing(item):
-                item.sizing_mode = sizing_mode
-            col_children.append(item)
-        else:
-            raise ValueError("""Only LayoutDOM items can be inserted into a widget box. Tried to insert: %s of type %s""" % (item, type(item)))
-    return WidgetBox(children=col_children, sizing_mode=sizing_mode, **kwargs)
+    _handle_child_sizing(children, sizing_mode, widget="widget box")
+
+    return WidgetBox(children=children, sizing_mode=sizing_mode, **kwargs)
 
 
 def layout(*args, **kwargs):
@@ -216,7 +191,7 @@ def layout(*args, **kwargs):
     sizing_mode = kwargs.pop('sizing_mode', None)
     children = kwargs.pop('children', None)
 
-    children = _handle_children(*args, children=children)
+    children = _parse_children_arg(*args, children=children)
 
     # Make the grid
     return _create_grid(children, sizing_mode, **kwargs)
@@ -286,7 +261,7 @@ def gridplot(children, sizing_mode=None, toolbar_location='above', ncols=None,
         if not hasattr(Location, toolbar_location):
             raise ValueError("Invalid value of toolbar_location: %s" % toolbar_location)
 
-    children = _handle_children(children=children)
+    children = _parse_children_arg(children=children)
     if ncols:
         if any(isinstance(child, list) for child in children):
             raise ValueError("Cannot provide a nested list when using ncols")
@@ -297,7 +272,7 @@ def gridplot(children, sizing_mode=None, toolbar_location='above', ncols=None,
         children = []
 
     # Make the grid
-    tools = []
+    toolbars = []
     items = []
 
     for y, row in enumerate(children):
@@ -307,7 +282,7 @@ def gridplot(children, sizing_mode=None, toolbar_location='above', ncols=None,
             elif isinstance(item, LayoutDOM):
                 if merge_tools:
                     for plot in item.select(dict(type=Plot)):
-                        tools += plot.toolbar.tools
+                        toolbars.append(plot.toolbar)
                         plot.toolbar_location = None
 
                 if isinstance(item, Plot):
@@ -327,7 +302,8 @@ def gridplot(children, sizing_mode=None, toolbar_location='above', ncols=None,
         return GridBox(children=items, sizing_mode=sizing_mode)
 
     grid = GridBox(children=items)
-    proxy = ProxyToolbar(tools=tools, **toolbar_options)
+    tools = sum([ toolbar.tools for toolbar in toolbars ], [])
+    proxy = ProxyToolbar(toolbars=toolbars, tools=tools, **toolbar_options)
     toolbar = ToolbarBox(toolbar=proxy, toolbar_location=toolbar_location)
 
     if toolbar_location == 'above':
@@ -415,7 +391,7 @@ def grid(children=[], sizing_mode=None, nrows=None, ncols=None):
                     return Grid(0, 0, [])
 
                 nrows = lcm(*[ child.nrows for child in children ])
-                ncols = sum([ child.ncols for child in children ])
+                ncols = sum(child.ncols for child in children)
 
                 items = []
                 offset = 0
@@ -433,7 +409,7 @@ def grid(children=[], sizing_mode=None, nrows=None, ncols=None):
                 if not children:
                     return Grid(0, 0, [])
 
-                nrows = sum([ child.nrows for child in children ])
+                nrows = sum(child.nrows for child in children)
                 ncols = lcm(*[ child.ncols for child in children ])
 
                 items = []
@@ -486,7 +462,7 @@ def grid(children=[], sizing_mode=None, nrows=None, ncols=None):
                 return item
 
         layout = traverse(children, top_level=True)
-    elif isinstance(children, string_types):
+    elif isinstance(children, str):
         raise NotImplementedError
     else:
         raise ValueError("expected a list, string or model")
@@ -507,7 +483,7 @@ def grid(children=[], sizing_mode=None, nrows=None, ncols=None):
 # Dev API
 #-----------------------------------------------------------------------------
 
-class GridSpec(object):
+class GridSpec:
     """ Simplifies grid layout specification. """
 
     def __init__(self, nrows, ncols):
@@ -515,6 +491,9 @@ class GridSpec(object):
         self.ncols = ncols
         self._arrangement = {}
 
+        from .util.deprecation import deprecated
+        deprecated("'GridSpec' is deprecated and will be removed in Bokeh 3.0")
+
     def __setitem__(self, key, obj):
         k1, k2 = key
 
@@ -551,13 +530,13 @@ class GridSpec(object):
             self._arrangement[row1, col1] = obj
         elif row2 is None:
             for col in range(col1, col2):
-                self._arrangement[row1, col] = get_or_else(lambda: obj[col-col1], None)
+                self._arrangement[row1, col] = get_or_else(lambda: obj[col-col1], None) # lgtm [py/loop-variable-capture]
         elif col2 is None:
             for row in range(row1, row2):
-                self._arrangement[row, col1] = get_or_else(lambda: obj[row-row1], None)
+                self._arrangement[row, col1] = get_or_else(lambda: obj[row-row1], None) # lgtm [py/loop-variable-capture]
         else:
             for row, col in zip(range(row1, row2), range(col1, col2)):
-                self._arrangement[row, col] = get_or_else(lambda: obj[row-row1][col-col1], None)
+                self._arrangement[row, col] = get_or_else(lambda: obj[row-row1][col-col1], None) # lgtm [py/loop-variable-capture]
 
     def __iter__(self):
         array = [ [ None ]*self.ncols for _ in range(0, self.nrows) ]
@@ -572,7 +551,7 @@ class GridSpec(object):
 def _has_auto_sizing(item):
     return item.sizing_mode is None and item.width_policy == "auto" and item.height_policy == "auto"
 
-def _handle_children(*args, **kwargs):
+def _parse_children_arg(*args, **kwargs):
     children = kwargs.get('children')
 
     # Set-up Children from args or kwargs
@@ -589,6 +568,13 @@ def _handle_children(*args, **kwargs):
 
     return children
 
+def _handle_child_sizing(children, sizing_mode, *, widget):
+    for item in children:
+        if not isinstance(item, LayoutDOM):
+            raise ValueError(f"Only LayoutDOM items can be inserted into a {widget}. Tried to insert: {item} of type {type(item)}")
+        if sizing_mode is not None and _has_auto_sizing(item):
+            item.sizing_mode = sizing_mode
+
 
 def _create_grid(iterable, sizing_mode, layer=0, **kwargs):
     """Recursively create grid from input lists."""
