@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -11,34 +11,68 @@ Bokeh plots
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
 # Imports
 #-----------------------------------------------------------------------------
 
-# Standard library imports
-
-# External imports
-from six import string_types
-
 # Bokeh imports
-from ..core.enums import (AngleUnits, Dimension, FontStyle, LegendClickPolicy, LegendLocation,
-                          Orientation, RenderMode, SpatialUnits, VerticalAlign, TextAlign,
-                          TooltipAttachment)
+from ..core.enums import (
+    AngleUnits,
+    Dimension,
+    FontStyle,
+    LegendClickPolicy,
+    LegendLocation,
+    Orientation,
+    RenderMode,
+    SpatialUnits,
+    TextAlign,
+    TooltipAttachment,
+    VerticalAlign,
+)
 from ..core.has_props import abstract
-from ..core.properties import (Angle, AngleSpec, Auto, Bool, ColorSpec, Datetime, Dict, Either,
-                               Enum, Float, FontSizeSpec, Include, Instance, Int, List, NumberSpec, Override,
-                               Seq, String, StringSpec, Tuple, UnitsSpec, value)
-from ..core.property_mixins import FillProps, LineProps, TextProps
+from ..core.properties import (
+    Angle,
+    AngleSpec,
+    Auto,
+    Bool,
+    ColorSpec,
+    Datetime,
+    Dict,
+    Either,
+    Enum,
+    Float,
+    FontSizeSpec,
+    Include,
+    Instance,
+    Int,
+    List,
+    NumberSpec,
+    Override,
+    PropertyUnitsSpec,
+    Seq,
+    String,
+    StringSpec,
+    Tuple,
+    value,
+)
+from ..core.property_mixins import (
+    FillProps,
+    LineProps,
+    ScalarFillProps,
+    ScalarLineProps,
+    ScalarTextProps,
+    TextProps,
+)
 from ..core.validation import error
-from ..core.validation.errors import BAD_COLUMN_NAME, NON_MATCHING_DATA_SOURCES_ON_LEGEND_ITEM_RENDERERS
+from ..core.validation.errors import (
+    BAD_COLUMN_NAME,
+    NON_MATCHING_DATA_SOURCES_ON_LEGEND_ITEM_RENDERERS,
+)
 from ..model import Model
 from ..util.serialization import convert_datetime_type
-
 from .formatters import BasicTickFormatter, TickFormatter
 from .mappers import ContinuousColorMapper
 from .renderers import GlyphRenderer, Renderer
@@ -123,8 +157,8 @@ class LegendItem(Model):
 
     '''
     def __init__(self, *args, **kwargs):
-        super(LegendItem, self).__init__(*args, **kwargs)
-        if isinstance(self.label, string_types):
+        super().__init__(*args, **kwargs)
+        if isinstance(self.label, str):
             # Allow convenience of setting label as a string
             self.label = value(self.label)
 
@@ -169,6 +203,8 @@ class LegendItem(Model):
 class Legend(Annotation):
     ''' Render informational legends for a plot.
 
+    See :ref:`userguide_plotting_legends` for information on plotting legends.
+
     '''
 
     location = Either(Enum(LegendLocation), Tuple(Float, Float), default="top_right", help="""
@@ -187,11 +223,11 @@ class Legend(Annotation):
     The title text to render.
     """)
 
-    title_props = Include(TextProps, help="""
+    title_props = Include(ScalarTextProps, help="""
     The %s values for the title text.
     """)
 
-    title_text_font_size = Override(default={'value': "10pt"})
+    title_text_font_size = Override(default="13px")
 
     title_text_font_style = Override(default="italic")
 
@@ -199,7 +235,7 @@ class Legend(Annotation):
     The distance (in pixels) to separate the title from the legend.
     """)
 
-    border_props = Include(LineProps, help="""
+    border_props = Include(ScalarLineProps, help="""
     The %s for the legend border outline.
     """)
 
@@ -207,11 +243,11 @@ class Legend(Annotation):
 
     border_line_alpha = Override(default=0.5)
 
-    background_props = Include(FillProps, help="""
+    background_props = Include(ScalarFillProps, help="""
     The %s for the legend background style.
     """)
 
-    inactive_props = Include(FillProps, help="""
+    inactive_props = Include(ScalarFillProps, help="""
     The %s for the legend item style when inactive. These control an overlay
     on the item that can be used to obscure it when the corresponding glyph
     is inactive (e.g. by making it semi-transparent).
@@ -229,13 +265,13 @@ class Legend(Annotation):
 
     inactive_fill_alpha = Override(default=0.7)
 
-    label_props = Include(TextProps, help="""
+    label_props = Include(ScalarTextProps, help="""
     The %s for the legend labels.
     """)
 
     label_text_baseline = Override(default='middle')
 
-    label_text_font_size = Override(default={'value': '10pt'})
+    label_text_font_size = Override(default='13px')
 
     label_standoff = Int(5, help="""
     The distance (in pixels) to separate the label from its associated glyph.
@@ -301,6 +337,8 @@ class Legend(Annotation):
 class ColorBar(Annotation):
     ''' Render a color bar based on a color mapper.
 
+    See :ref:`userguide_plotting_color_bars` for information on plotting color bars.
+
     '''
 
     location = Either(Enum(LegendLocation), Tuple(Float, Float),
@@ -335,11 +373,11 @@ class ColorBar(Annotation):
     The title text to render.
     """)
 
-    title_props = Include(TextProps, help="""
+    title_props = Include(ScalarTextProps, help="""
     The %s values for the title text.
     """)
 
-    title_text_font_size = Override(default={'value': "10pt"})
+    title_text_font_size = Override(default="13px")
 
     title_text_font_style = Override(default="italic")
 
@@ -379,7 +417,7 @@ class ColorBar(Annotation):
     Amount of padding (in pixels) between the color scale and color bar border.
     """)
 
-    major_label_props = Include(TextProps, help="""
+    major_label_props = Include(ScalarTextProps, help="""
     The %s of the major tick labels.
     """)
 
@@ -387,13 +425,13 @@ class ColorBar(Annotation):
 
     major_label_text_baseline = Override(default="middle")
 
-    major_label_text_font_size = Override(default={'value': "8pt"})
+    major_label_text_font_size = Override(default="11px")
 
     label_standoff = Int(5, help="""
     The distance (in pixels) to separate the tick labels from the color bar.
     """)
 
-    major_tick_props = Include(LineProps, help="""
+    major_tick_props = Include(ScalarLineProps, help="""
     The %s of the major ticks.
     """)
 
@@ -409,7 +447,7 @@ class ColorBar(Annotation):
     main plot area.
     """)
 
-    minor_tick_props = Include(LineProps, help="""
+    minor_tick_props = Include(ScalarLineProps, help="""
     The %s of the minor ticks.
     """)
 
@@ -425,19 +463,19 @@ class ColorBar(Annotation):
     main plot area.
     """)
 
-    bar_props = Include(LineProps, help="""
+    bar_props = Include(ScalarLineProps, help="""
     The %s for the color scale bar outline.
     """)
 
     bar_line_color = Override(default=None)
 
-    border_props = Include(LineProps, help="""
+    border_props = Include(ScalarLineProps, help="""
     The %s for the color bar border outline.
     """)
 
     border_line_color = Override(default=None)
 
-    background_props = Include(FillProps, help="""
+    background_props = Include(ScalarFillProps, help="""
     The %s for the color bar background style.
     """)
 
@@ -446,7 +484,9 @@ class ColorBar(Annotation):
     background_fill_alpha = Override(default=0.95)
 
 class Arrow(Annotation):
-    ''' Render an arrow as an annotation.
+    ''' Render arrows as an annotation.
+
+    See :ref:`userguide_plotting_arrows` for information on plotting arrows.
 
     '''
 
@@ -492,19 +532,11 @@ class Arrow(Annotation):
     Local data source to use when rendering annotations on the plot.
     """)
 
-    x_range_name = String('default', help="""
-    A particular (named) x-range to use for computing screen locations when
-    rendering annotations on the plot. If unset, use the default x-range.
-    """)
-
-    y_range_name = String('default', help="""
-    A particular (named) y-range to use for computing screen locations when
-    rendering annotations on the plot. If unset, use the default y-range.
-    """)
-
 class BoxAnnotation(Annotation):
     ''' Render a shaded rectangular region as an annotation.
 
+    See :ref:`userguide_plotting_box_annotations` for information on plotting box annotations.
+
     '''
 
     left = Either(Auto, NumberSpec(), default=None, help="""
@@ -555,17 +587,7 @@ class BoxAnnotation(Annotation):
     by default.
     """)
 
-    x_range_name = String('default', help="""
-    A particular (named) x-range to use for computing screen locations when
-    rendering box annotations on the plot. If unset, use the default x-range.
-    """)
-
-    y_range_name = String('default', help="""
-    A particular (named) y-range to use for computing screen locations when
-    rendering box annotations on the plot. If unset, use the default y-range.
-    """)
-
-    line_props = Include(LineProps, use_prefix=False, help="""
+    line_props = Include(ScalarLineProps, use_prefix=False, help="""
     The %s values for the box.
     """)
 
@@ -573,7 +595,7 @@ class BoxAnnotation(Annotation):
 
     line_color = Override(default="#cccccc")
 
-    fill_props = Include(FillProps, use_prefix=False, help="""
+    fill_props = Include(ScalarFillProps, use_prefix=False, help="""
     The %s values for the box.
     """)
 
@@ -585,6 +607,9 @@ class BoxAnnotation(Annotation):
     Specifies whether the box is rendered as a canvas element or as an
     css element overlaid on the canvas. The default mode is "canvas".
 
+    .. note:
+        This property is deprecated and will be removed in bokeh 3.0.
+
     .. warning::
         The line_dash and line_dash_offset attributes aren't supported if
         the render_mode is set to "css"
@@ -594,16 +619,18 @@ class BoxAnnotation(Annotation):
 class Band(Annotation):
     ''' Render a filled area band along a dimension.
 
+    See :ref:`userguide_plotting_bands` for information on plotting bands.
+
     '''
-    lower = UnitsSpec(default=None, units_type=Enum(SpatialUnits), units_default="data", help="""
+    lower = PropertyUnitsSpec(default=None, units_type=Enum(SpatialUnits), units_default="data", help="""
     The coordinates of the lower portion of the filled area band.
     """)
 
-    upper = UnitsSpec(default=None, units_type=Enum(SpatialUnits), units_default="data", help="""
+    upper = PropertyUnitsSpec(default=None, units_type=Enum(SpatialUnits), units_default="data", help="""
     The coordinates of the upper portion of the filled area band.
     """)
 
-    base = UnitsSpec(default=None, units_type=Enum(SpatialUnits), units_default="data", help="""
+    base = PropertyUnitsSpec(default=None, units_type=Enum(SpatialUnits), units_default="data", help="""
     The orthogonal coordinates of the upper and lower values.
     """)
 
@@ -616,17 +643,7 @@ class Band(Annotation):
     Local data source to use when rendering annotations on the plot.
     """)
 
-    x_range_name = String('default', help="""
-    A particular (named) x-range to use for computing screen locations when
-    rendering annotations on the plot. If unset, use the default x-range.
-    """)
-
-    y_range_name = String('default', help="""
-    A particular (named) y-range to use for computing screen locations when
-    rendering annotations on the plot. If unset, use the default y-range.
-    """)
-
-    line_props = Include(LineProps, use_prefix=False, help="""
+    line_props = Include(ScalarLineProps, use_prefix=False, help="""
     The %s values for the band.
     """)
 
@@ -634,7 +651,7 @@ class Band(Annotation):
 
     line_color = Override(default="#cccccc")
 
-    fill_props = Include(FillProps, use_prefix=False, help="""
+    fill_props = Include(ScalarFillProps, use_prefix=False, help="""
     The %s values for the band.
     """)
 
@@ -658,6 +675,8 @@ class Label(TextAnnotation):
     appearance of the text, its background, as well as the rectangular bounding
     box border.
 
+    See :ref:`userguide_plotting_labels` for information on plotting labels.
+
     '''
 
     x = Float(help="""
@@ -716,35 +735,22 @@ class Label(TextAnnotation):
     distance in screen units from a given data position.
     """)
 
-    # TODO (bev) these should probably not be dataspec properties
-    text_props = Include(TextProps, use_prefix=False, help="""
+    text_props = Include(ScalarTextProps, use_prefix=False, help="""
     The %s values for the text.
     """)
 
-    # TODO (bev) these should probably not be dataspec properties
-    background_props = Include(FillProps, use_prefix=True, help="""
+    background_props = Include(ScalarFillProps, use_prefix=True, help="""
     The %s values for the text bounding box.
     """)
 
     background_fill_color = Override(default=None)
 
-    # TODO (bev) these should probably not be dataspec properties
-    border_props = Include(LineProps, use_prefix=True, help="""
+    border_props = Include(ScalarLineProps, use_prefix=True, help="""
     The %s values for the text bounding box.
     """)
 
     border_line_color = Override(default=None)
 
-    x_range_name = String('default', help="""
-    A particular (named) x-range to use for computing screen location when
-    rendering an annotation on the plot. If unset, use the default x-range.
-    """)
-
-    y_range_name = String('default', help="""
-    A particular (named) y-range to use for computing screen location when
-    rendering an annotation on the plot. If unset, use the default y-range.
-    """)
-
 class LabelSet(TextAnnotation):
     ''' Render multiple text labels as annotations.
 
@@ -836,16 +842,6 @@ class LabelSet(TextAnnotation):
     Local data source to use when rendering annotations on the plot.
     """)
 
-    x_range_name = String('default', help="""
-    A particular (named) x-range to use for computing screen locations when
-    rendering annotations on the plot. If unset, use the default x-range.
-    """)
-
-    y_range_name = String('default', help="""
-    A particular (named) y-range to use for computing screen locations when
-    rendering annotations on the plot. If unset, use the default y-range.
-    """)
-
 class PolyAnnotation(Annotation):
     ''' Render a shaded polygonal region as an annotation.
 
@@ -869,17 +865,7 @@ class PolyAnnotation(Annotation):
     by default.
     """)
 
-    x_range_name = String('default', help="""
-    A particular (named) x-range to use for computing screen locations when
-    rendering box annotations on the plot. If unset, use the default x-range.
-    """)
-
-    y_range_name = String('default', help="""
-    A particular (named) y-range to use for computing screen locations when
-    rendering box annotations on the plot. If unset, use the default y-range.
-    """)
-
-    line_props = Include(LineProps, use_prefix=False, help="""
+    line_props = Include(ScalarLineProps, use_prefix=False, help="""
     The %s values for the polygon.
     """)
 
@@ -887,7 +873,7 @@ class PolyAnnotation(Annotation):
 
     line_color = Override(default="#cccccc")
 
-    fill_props = Include(FillProps, use_prefix=False, help="""
+    fill_props = Include(ScalarFillProps, use_prefix=False, help="""
     The %s values for the polygon.
     """)
 
@@ -898,6 +884,8 @@ class PolyAnnotation(Annotation):
 class Slope(Annotation):
     """ Render a sloped line as an annotation.
 
+    See :ref:`userguide_plotting_slope` for information on plotting slopes.
+
     """
 
     gradient = Float(help="""
@@ -908,23 +896,15 @@ class Slope(Annotation):
     The y intercept of the line, in data units
     """)
 
-    x_range_name = String('default', help="""
-    A particular (named) x-range to use for computing screen locations when
-    rendering annotations on the plot. If unset, use the default x-range.
-    """)
-
-    y_range_name = String('default', help="""
-    A particular (named) y-range to use for computing screen locations when
-    rendering annotations on the plot. If unset, use the default y-range.
-    """)
-
-    line_props = Include(LineProps, use_prefix=False, help="""
+    line_props = Include(ScalarLineProps, use_prefix=False, help="""
     The %s values for the line.
     """)
 
 class Span(Annotation):
     """ Render a horizontal or vertical line span.
 
+    See :ref:`userguide_plotting_spans` for information on plotting spans.
+
     """
 
     location = Float(help="""
@@ -944,33 +924,28 @@ class Span(Annotation):
     to "height" (``y`` direction) or "width" (``x`` direction).
     """)
 
-    x_range_name = String('default', help="""
-    A particular (named) x-range to use for computing screen locations when
-    rendering annotations on the plot. If unset, use the default x-range.
-    """)
-
-    y_range_name = String('default', help="""
-    A particular (named) y-range to use for computing screen locations when
-    rendering annotations on the plot. If unset, use the default y-range.
-    """)
-
     render_mode = Enum(RenderMode, default="canvas", help="""
     Specifies whether the span is rendered as a canvas element or as a
     CSS element overlaid on the canvas. The default mode is "canvas".
 
+    .. note:
+        This property is deprecated and will be removed in bokeh 3.0.
+
     .. warning::
         The line_dash and line_dash_offset attributes aren't supported if
         the render_mode is set to "css"
 
     """)
 
-    line_props = Include(LineProps, use_prefix=False, help="""
+    line_props = Include(ScalarLineProps, use_prefix=False, help="""
     The %s values for the span.
     """)
 
 class Title(TextAnnotation):
     ''' Render a single title box as an annotation.
 
+    See :ref:`userguide_plotting_titles` for information on plotting titles.
+
     '''
 
     text = String(help="""
@@ -1008,7 +983,7 @@ class Title(TextAnnotation):
 
     """)
 
-    text_font_size = FontSizeSpec(default=value("10pt"))
+    text_font_size = FontSizeSpec(default="13px")
 
     text_font_style = Enum(FontStyle, default="bold", help="""
     A style to use for rendering text.
@@ -1043,13 +1018,13 @@ class Title(TextAnnotation):
 
     """)
 
-    background_props = Include(FillProps, use_prefix=True, help="""
+    background_props = Include(ScalarFillProps, use_prefix=True, help="""
     The %s values for the text bounding box.
     """)
 
     background_fill_color = Override(default=None)
 
-    border_props = Include(LineProps, use_prefix=True, help="""
+    border_props = Include(ScalarLineProps, use_prefix=True, help="""
     The %s values for the text bounding box.
     """)
 
@@ -1073,6 +1048,10 @@ class Tooltip(Annotation):
 
     inner_only = Bool(default=True, help="""
     Whether to display outside a central plot frame area.
+
+    .. note:
+        This property is deprecated and will be removed in bokeh 3.0.
+
     """)
 
     show_arrow = Bool(default=True, help="""
@@ -1082,9 +1061,11 @@ class Tooltip(Annotation):
 class Whisker(Annotation):
     ''' Render a whisker along a dimension.
 
+    See :ref:`userguide_plotting_whiskers` for information on plotting whiskers.
+
     '''
 
-    lower = UnitsSpec(default=None, units_type=Enum(SpatialUnits), units_default="data", help="""
+    lower = PropertyUnitsSpec(default=None, units_type=Enum(SpatialUnits), units_default="data", help="""
     The coordinates of the lower end of the whiskers.
     """)
 
@@ -1092,7 +1073,7 @@ class Whisker(Annotation):
     Instance of ``ArrowHead``.
     """)
 
-    upper = UnitsSpec(default=None, units_type=Enum(SpatialUnits), units_default="data", help="""
+    upper = PropertyUnitsSpec(default=None, units_type=Enum(SpatialUnits), units_default="data", help="""
     The coordinates of the upper end of the whiskers.
     """)
 
@@ -1100,7 +1081,7 @@ class Whisker(Annotation):
     Instance of ``ArrowHead``.
     """)
 
-    base = UnitsSpec(default=None, units_type=Enum(SpatialUnits), units_default="data", help="""
+    base = PropertyUnitsSpec(default=None, units_type=Enum(SpatialUnits), units_default="data", help="""
     The orthogonal coordinates of the upper and lower values.
     """)
 
@@ -1113,16 +1094,6 @@ class Whisker(Annotation):
     Local data source to use when rendering annotations on the plot.
     """)
 
-    x_range_name = String('default', help="""
-    A particular (named) x-range to use for computing screen locations when
-    rendering annotations on the plot. If unset, use the default x-range.
-    """)
-
-    y_range_name = String('default', help="""
-    A particular (named) y-range to use for computing screen locations when
-    rendering annotations on the plot. If unset, use the default y-range.
-    """)
-
     line_props = Include(LineProps, use_prefix=False, help="""
     The %s values for the whisker body.
     """)
