@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -48,27 +48,49 @@ All these glyphs share a minimal common interface through their base class
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
 # Imports
 #-----------------------------------------------------------------------------
 
-# Standard library imports
-
-# External imports
-
 # Bokeh imports
 from ..core.enums import Anchor, Direction, StepMode
-from ..core.has_props import abstract
-from ..core.properties import (AngleSpec, Bool, DistanceSpec, Enum, Float, String,
-                               Include, Instance, Int, NumberSpec, Override, StringSpec)
-from ..core.property_mixins import FillProps, HatchProps, LineProps, ScalarFillProps, ScalarHatchProps, ScalarLineProps, TextProps
-from ..model import Model
-
+from ..core.properties import (
+    AngleSpec,
+    Bool,
+    DistanceSpec,
+    Enum,
+    Float,
+    Include,
+    Instance,
+    Int,
+    NumberSpec,
+    Override,
+    String,
+    StringSpec,
+)
+from ..core.property.dataspec import field
+from ..core.property_mixins import (
+    FillProps,
+    HatchProps,
+    LineProps,
+    ScalarFillProps,
+    ScalarHatchProps,
+    ScalarLineProps,
+    TextProps,
+)
+from ..util.deprecation import deprecated
+from .glyph import (
+    ConnectedXYGlyph,
+    FillGlyph,
+    Glyph,
+    HatchGlyph,
+    LineGlyph,
+    TextGlyph,
+    XYGlyph,
+)
 from .mappers import ColorMapper, LinearColorMapper
 
 #-----------------------------------------------------------------------------
@@ -80,6 +102,7 @@ __all__ = (
     'Annulus',
     'Arc',
     'Bezier',
+    'ConnectedXYGlyph',
     'Ellipse',
     'Glyph',
     'HArea',
@@ -111,19 +134,7 @@ __all__ = (
 # General API
 #-----------------------------------------------------------------------------
 
-@abstract
-class Glyph(Model):
-    ''' Base class for all glyph models.
-
-    '''
-
-@abstract
-class XYGlyph(Glyph):
-    ''' Base class of glyphs with `x` and `y` coordinate attributes.
-
-    '''
-
-class AnnularWedge(XYGlyph):
+class AnnularWedge(XYGlyph, LineGlyph, FillGlyph):
     ''' Render annular wedges.
 
     '''
@@ -134,11 +145,11 @@ class AnnularWedge(XYGlyph):
     # functions derived from this class
     _args = ('x', 'y', 'inner_radius', 'outer_radius', 'start_angle', 'end_angle', 'direction')
 
-    x = NumberSpec(help="""
+    x = NumberSpec(default=field("x"), help="""
     The x-coordinates of the center of the annular wedges.
     """)
 
-    y = NumberSpec(help="""
+    y = NumberSpec(default=field("y"), help="""
     The y-coordinates of the center of the annular wedges.
     """)
 
@@ -170,7 +181,7 @@ class AnnularWedge(XYGlyph):
     The %s values for the annular wedges.
     """)
 
-class Annulus(XYGlyph):
+class Annulus(XYGlyph, LineGlyph, FillGlyph):
     ''' Render annuli.
 
     '''
@@ -181,11 +192,11 @@ class Annulus(XYGlyph):
     # functions derived from this class
     _args = ('x', 'y', 'inner_radius', 'outer_radius')
 
-    x = NumberSpec(help="""
+    x = NumberSpec(default=field("x"), help="""
     The x-coordinates of the center of the annuli.
     """)
 
-    y = NumberSpec(help="""
+    y = NumberSpec(default=field("y"), help="""
     The y-coordinates of the center of the annuli.
     """)
 
@@ -205,7 +216,7 @@ class Annulus(XYGlyph):
     The %s values for the annuli.
     """)
 
-class Arc(XYGlyph):
+class Arc(XYGlyph, LineGlyph):
     ''' Render arcs.
 
     '''
@@ -216,11 +227,11 @@ class Arc(XYGlyph):
     # functions derived from this class
     _args = ('x', 'y', 'radius', 'start_angle', 'end_angle', 'direction')
 
-    x = NumberSpec(help="""
+    x = NumberSpec(default=field("x"), help="""
     The x-coordinates of the center of the arcs.
     """)
 
-    y = NumberSpec(help="""
+    y = NumberSpec(default=field("y"), help="""
     The y-coordinates of the center of the arcs.
     """)
 
@@ -244,7 +255,7 @@ class Arc(XYGlyph):
     The %s values for the arcs.
     """)
 
-class Bezier(Glyph):
+class Bezier(LineGlyph):
     ''' Render Bezier curves.
 
     For more information consult the `Wikipedia article for Bezier curve`_.
@@ -257,37 +268,37 @@ class Bezier(Glyph):
 
     # a canonical order for positional args that can be used for any
     # functions derived from this class
-    _args = ('x0', 'y0', 'x1', 'y1', 'cx0', 'cy0', 'cx1', 'cy1')
+    _args = ("x0", "y0", "x1", "y1", "cx0", "cy0", "cx1", "cy1")
 
-    x0 = NumberSpec(help="""
+    x0 = NumberSpec(default=field("x0"), help="""
     The x-coordinates of the starting points.
     """)
 
-    y0 = NumberSpec(help="""
+    y0 = NumberSpec(default=field("y0"), help="""
     The y-coordinates of the starting points.
     """)
 
-    x1 = NumberSpec(help="""
+    x1 = NumberSpec(default=field("x1"), help="""
     The x-coordinates of the ending points.
     """)
 
-    y1 = NumberSpec(help="""
+    y1 = NumberSpec(default=field("y1"), help="""
     The y-coordinates of the ending points.
     """)
 
-    cx0 = NumberSpec(help="""
+    cx0 = NumberSpec(default=field("cx0"), help="""
     The x-coordinates of first control points.
     """)
 
-    cy0 = NumberSpec(help="""
+    cy0 = NumberSpec(default=field("cy0"), help="""
     The y-coordinates of first control points.
     """)
 
-    cx1 = NumberSpec(help="""
+    cx1 = NumberSpec(default=field("cx1"), help="""
     The x-coordinates of second control points.
     """)
 
-    cy1 = NumberSpec(help="""
+    cy1 = NumberSpec(default=field("cy1"), help="""
     The y-coordinates of second control points.
     """)
 
@@ -295,7 +306,7 @@ class Bezier(Glyph):
     The %s values for the Bezier curves.
     """)
 
-class Ellipse(XYGlyph):
+class Ellipse(XYGlyph, LineGlyph, FillGlyph):
     ''' Render ellipses.
 
     '''
@@ -306,11 +317,11 @@ class Ellipse(XYGlyph):
     # functions derived from this class
     _args = ('x', 'y', 'width', 'height', 'angle')
 
-    x = NumberSpec(help="""
+    x = NumberSpec(default=field("x"), help="""
     The x-coordinates of the centers of the ellipses.
     """)
 
-    y = NumberSpec(help="""
+    y = NumberSpec(default=field("y"), help="""
     The y-coordinates of the centers of the ellipses.
     """)
 
@@ -334,7 +345,7 @@ class Ellipse(XYGlyph):
     The %s values for the ovals.
     """)
 
-class HArea(Glyph):
+class HArea(FillGlyph, HatchGlyph, LineGlyph):
     ''' Render a horizontally directed area between two equal length sequences
     of x-coordinates with the same y-coordinates.
 
@@ -346,27 +357,27 @@ class HArea(Glyph):
     # functions derived from this class
     _args = ('x1', 'x2', 'y')
 
-    x1 = NumberSpec(help="""
+    x1 = NumberSpec(default=field("x1"), help="""
     The x-coordinates for the points of one side of the area.
     """)
 
-    x2 = NumberSpec(help="""
+    x2 = NumberSpec(default=field("x2"), help="""
     The x-coordinates for the points of the other side of the area.
     """)
 
-    y = NumberSpec(help="""
+    y = NumberSpec(default=field("y"), help="""
     The y-coordinates for the points of the area.
     """)
 
     fill_props = Include(ScalarFillProps, use_prefix=False, help="""
-    The %s values for the patch.
+    The %s values for the horizontal directed area.
     """)
 
     hatch_props = Include(HatchProps, use_prefix=False, help="""
     The %s values for the horizontal bars.
     """)
 
-class HBar(Glyph):
+class HBar(LineGlyph, FillGlyph, HatchGlyph):
     ''' Render horizontal bars, given a center coordinate, ``height`` and
     (``left``, ``right``) coordinates.
 
@@ -378,7 +389,7 @@ class HBar(Glyph):
     # functions derived from this class
     _args = ('y', 'height', 'right', 'left')
 
-    y = NumberSpec(help="""
+    y = NumberSpec(default=field("y"), help="""
     The y-coordinates of the centers of the horizontal bars.
     """)
 
@@ -406,7 +417,7 @@ class HBar(Glyph):
     The %s values for the horizontal bars.
     """)
 
-class HexTile(Glyph):
+class HexTile(LineGlyph, FillGlyph):
     ''' Render horizontal tiles on a regular hexagonal grid.
 
     '''
@@ -483,11 +494,11 @@ class Image(XYGlyph):
             mapper = LinearColorMapper(palette)
             kwargs['color_mapper'] = mapper
 
-        super(Image, self).__init__(**kwargs)
+        super().__init__(**kwargs)
 
     # a canonical order for positional args that can be used for any
     # functions derived from this class
-    _args = ('image', 'x', 'y', 'dw', 'dh', 'global_alpha', 'dilate')
+    _args = ('image', 'x', 'y', 'dw', 'dh', 'dilate')
 
     # a hook to specify any additional kwargs handled by an initializer
     _extra_kws = {
@@ -501,11 +512,11 @@ class Image(XYGlyph):
     The arrays of scalar data for the images to be colormapped.
     """)
 
-    x = NumberSpec(help="""
+    x = NumberSpec(default=field("x"), help="""
     The x-coordinates to locate the image anchors.
     """)
 
-    y = NumberSpec(help="""
+    y = NumberSpec(default=field("y"), help="""
     The y-coordinates to locate the image anchors.
     """)
 
@@ -556,17 +567,17 @@ class ImageRGBA(XYGlyph):
 
     # a canonical order for positional args that can be used for any
     # functions derived from this class
-    _args = ('image', 'x', 'y', 'dw', 'dh', 'global_alpha', 'dilate')
+    _args = ('image', 'x', 'y', 'dw', 'dh', 'dilate')
 
     image = NumberSpec(help="""
     The arrays of RGBA data for the images.
     """)
 
-    x = NumberSpec(help="""
+    x = NumberSpec(default=field("x"), help="""
     The x-coordinates to locate the image anchors.
     """)
 
-    y = NumberSpec(help="""
+    y = NumberSpec(default=field("y"), help="""
     The y-coordinates to locate the image anchors.
     """)
 
@@ -611,7 +622,7 @@ class ImageURL(XYGlyph):
 
     # a canonical order for positional args that can be used for any
     # functions derived from this class
-    _args = ('url', 'x', 'y', 'w', 'h', 'angle', 'global_alpha', 'dilate')
+    _args = ('url', 'x', 'y', 'w', 'h', 'angle', 'dilate')
 
     url = StringSpec(default=None, help="""
     The URLs to retrieve images from.
@@ -621,11 +632,11 @@ class ImageURL(XYGlyph):
         the client.
     """)
 
-    x = NumberSpec(help="""
+    x = NumberSpec(default=field("x"), help="""
     The x-coordinates to locate the image anchors.
     """)
 
-    y = NumberSpec(help="""
+    y = NumberSpec(default=field("y"), help="""
     The y-coordinates to locate the image anchors.
     """)
 
@@ -675,7 +686,7 @@ class ImageURL(XYGlyph):
     specified URL. Default is zero ms.
     """)
 
-class Line(XYGlyph):
+class Line(ConnectedXYGlyph, LineGlyph):
     ''' Render a single line.
 
     The ``Line`` glyph is different from most other glyphs in that the vector
@@ -688,11 +699,11 @@ class Line(XYGlyph):
 
     __example__ = "examples/reference/models/Line.py"
 
-    x = NumberSpec(help="""
+    x = NumberSpec(default=field("x"), help="""
     The x-coordinates for the points of the line.
     """)
 
-    y = NumberSpec(help="""
+    y = NumberSpec(default=field("y"), help="""
     The y-coordinates for the points of the line.
     """)
 
@@ -700,7 +711,7 @@ class Line(XYGlyph):
     The %s values for the line.
     """)
 
-class MultiLine(Glyph):
+class MultiLine(LineGlyph):
     ''' Render several lines.
 
     The data for the ``MultiLine`` glyph is different in that the vector of
@@ -714,11 +725,11 @@ class MultiLine(Glyph):
     # functions derived from this class
     _args = ('xs', 'ys')
 
-    xs = NumberSpec(help="""
+    xs = NumberSpec(default=field("xs"), help="""
     The x-coordinates for all the lines, given as a "list of lists".
     """)
 
-    ys = NumberSpec(help="""
+    ys = NumberSpec(default=field("ys"), help="""
     The y-coordinates for all the lines, given as a "list of lists".
     """)
 
@@ -726,12 +737,16 @@ class MultiLine(Glyph):
     The %s values for the lines.
     """)
 
-class MultiPolygons(Glyph):
+class MultiPolygons(LineGlyph, FillGlyph, HatchGlyph):
     ''' Render several MultiPolygon.
 
     Modeled on geoJSON - the data for the ``MultiPolygons`` glyph is
     different in that the vector of values is not a vector of scalars.
     Rather, it is a "list of lists of lists of lists".
+
+    During box selection only multi-polygons entirely contained in the
+    selection box will be included.
+
     '''
 
     __example__ = "examples/reference/models/MultiPolygons.py"
@@ -740,7 +755,7 @@ class MultiPolygons(Glyph):
     # functions derived from this class
     _args = ('xs', 'ys')
 
-    xs = NumberSpec(help="""
+    xs = NumberSpec(default=field("xs"), help="""
     The x-coordinates for all the patches, given as a nested list.
 
     .. note::
@@ -749,7 +764,7 @@ class MultiPolygons(Glyph):
         one exterior ring optionally followed by ``m`` interior rings (holes).
     """)
 
-    ys = NumberSpec(help="""
+    ys = NumberSpec(default=field("ys"), help="""
     The y-coordinates for all the patches, given as a "list of lists".
 
     .. note::
@@ -770,7 +785,7 @@ class MultiPolygons(Glyph):
     The %s values for the patches.
     """)
 
-class Oval(XYGlyph):
+class Oval(XYGlyph, LineGlyph, FillGlyph):
     ''' Render ovals.
 
     This glyph renders ovals using Bezier curves, which are similar,
@@ -779,17 +794,21 @@ class Oval(XYGlyph):
 
     '''
 
+    def __init__(self, **kwargs):
+        deprecated("'Oval' is deprecated and will be removed in Bokeh 3.0, use the Ellipse glyph instead")
+        super().__init__(**kwargs)
+
     __example__ = "examples/reference/models/Oval.py"
 
     # a canonical order for positional args that can be used for any
     # functions derived from this class
     _args = ('x', 'y', 'width', 'height', 'angle')
 
-    x = NumberSpec(help="""
+    x = NumberSpec(default=field("x"), help="""
     The x-coordinates of the centers of the ovals.
     """)
 
-    y = NumberSpec(help="""
+    y = NumberSpec(default=field("y"), help="""
     The y-coordinates of the centers of the ovals.
     """)
 
@@ -813,7 +832,7 @@ class Oval(XYGlyph):
     The %s values for the ovals.
     """)
 
-class Patch(XYGlyph):
+class Patch(ConnectedXYGlyph, LineGlyph, FillGlyph, HatchGlyph):
     ''' Render a single patch.
 
     The ``Patch`` glyph is different from most other glyphs in that the vector
@@ -827,7 +846,7 @@ class Patch(XYGlyph):
     # functions derived from this class
     _args = ('x', 'y')
 
-    x = NumberSpec(help="""
+    x = NumberSpec(default=field("x"), help="""
     The x-coordinates for the points of the patch.
 
     .. note::
@@ -836,7 +855,7 @@ class Patch(XYGlyph):
         values in the sequence.
     """)
 
-    y = NumberSpec(help="""
+    y = NumberSpec(default=field("y"), help="""
     The y-coordinates for the points of the patch.
 
     .. note::
@@ -857,12 +876,15 @@ class Patch(XYGlyph):
     The %s values for the patch.
     """)
 
-class Patches(Glyph):
+class Patches(LineGlyph, FillGlyph, HatchGlyph):
     ''' Render several patches.
 
     The data for the ``Patches`` glyph is different in that the vector of
     values is not a vector of scalars. Rather, it is a "list of lists".
 
+    During box selection only patches entirely contained in the
+    selection box will be included.
+
     '''
 
     __example__ = "examples/reference/models/Patches.py"
@@ -871,7 +893,7 @@ class Patches(Glyph):
     # functions derived from this class
     _args = ('xs', 'ys')
 
-    xs = NumberSpec(help="""
+    xs = NumberSpec(default=field("xs"), help="""
     The x-coordinates for all the patches, given as a "list of lists".
 
     .. note::
@@ -880,7 +902,7 @@ class Patches(Glyph):
         values in the sublists.
     """)
 
-    ys = NumberSpec(help="""
+    ys = NumberSpec(default=field("ys"), help="""
     The y-coordinates for all the patches, given as a "list of lists".
 
     .. note::
@@ -901,7 +923,7 @@ class Patches(Glyph):
     The %s values for the patches.
     """)
 
-class Quad(Glyph):
+class Quad(LineGlyph, FillGlyph, HatchGlyph):
     ''' Render axis-aligned quads.
 
     '''
@@ -940,7 +962,7 @@ class Quad(Glyph):
     The %s values for the horizontal bars.
     """)
 
-class Quadratic(Glyph):
+class Quadratic(LineGlyph):
     ''' Render parabolas.
 
     '''
@@ -949,29 +971,29 @@ class Quadratic(Glyph):
 
     # a canonical order for positional args that can be used for any
     # functions derived from this class
-    _args = ('x0', 'y0', 'x1', 'y1', 'cx', 'cy')
+    _args = ("x0", "y0", "x1", "y1", "cx", "cy")
 
-    x0 = NumberSpec(help="""
+    x0 = NumberSpec(default=field("x0"), help="""
     The x-coordinates of the starting points.
     """)
 
-    y0 = NumberSpec(help="""
+    y0 = NumberSpec(default=field("y0"), help="""
     The y-coordinates of the starting points.
     """)
 
-    x1 = NumberSpec(help="""
+    x1 = NumberSpec(default=field("x1"), help="""
     The x-coordinates of the ending points.
     """)
 
-    y1 = NumberSpec(help="""
+    y1 = NumberSpec(default=field("y1"), help="""
     The y-coordinates of the ending points.
     """)
 
-    cx = NumberSpec(help="""
+    cx = NumberSpec(default=field("cx"), help="""
     The x-coordinates of the control points.
     """)
 
-    cy = NumberSpec(help="""
+    cy = NumberSpec(default=field("cy"), help="""
     The y-coordinates of the control points.
     """)
 
@@ -979,7 +1001,7 @@ class Quadratic(Glyph):
     The %s values for the parabolas.
     """)
 
-class Ray(XYGlyph):
+class Ray(XYGlyph, LineGlyph):
     ''' Render rays.
 
     '''
@@ -990,11 +1012,11 @@ class Ray(XYGlyph):
     # functions derived from this class
     _args = ('x', 'y', 'length', 'angle')
 
-    x = NumberSpec(help="""
+    x = NumberSpec(default=field("x"), help="""
     The x-coordinates to start the rays.
     """)
 
-    y = NumberSpec(help="""
+    y = NumberSpec(default=field("y"), help="""
     The y-coordinates to start the rays.
     """)
 
@@ -1011,7 +1033,7 @@ class Ray(XYGlyph):
     The %s values for the rays.
     """)
 
-class Rect(XYGlyph):
+class Rect(XYGlyph, LineGlyph, FillGlyph):
     ''' Render rectangles.
 
     '''
@@ -1022,11 +1044,11 @@ class Rect(XYGlyph):
     # functions derived from this class
     _args = ('x', 'y', 'width', 'height', 'angle', 'dilate')
 
-    x = NumberSpec(help="""
+    x = NumberSpec(default=field("x"), help="""
     The x-coordinates of the centers of the rectangles.
     """)
 
-    y = NumberSpec(help="""
+    y = NumberSpec(default=field("y"), help="""
     The y-coordinates of the centers of the rectangles.
     """)
 
@@ -1059,7 +1081,7 @@ class Rect(XYGlyph):
     The %s values for the rectangles.
     """)
 
-class Segment(Glyph):
+class Segment(LineGlyph):
     ''' Render segments.
 
     '''
@@ -1070,19 +1092,19 @@ class Segment(Glyph):
     # functions derived from this class
     _args = ('x0', 'y0', 'x1', 'y1')
 
-    x0 = NumberSpec(help="""
+    x0 = NumberSpec(default=field("x0"), help="""
     The x-coordinates of the starting points.
     """)
 
-    y0 = NumberSpec(help="""
+    y0 = NumberSpec(default=field("y0"), help="""
     The y-coordinates of the starting points.
     """)
 
-    x1 = NumberSpec(help="""
+    x1 = NumberSpec(default=field("x1"), help="""
     The x-coordinates of the ending points.
     """)
 
-    y1 = NumberSpec(help="""
+    y1 = NumberSpec(default=field("y1"), help="""
     The y-coordinates of the ending points.
     """)
 
@@ -1090,7 +1112,7 @@ class Segment(Glyph):
     The %s values for the segments.
     """)
 
-class Step(XYGlyph):
+class Step(XYGlyph, LineGlyph):
     ''' Render step lines.
 
     Step levels can be draw before, after, or centered on each point, according
@@ -1107,11 +1129,11 @@ class Step(XYGlyph):
     # functions derived from this class
     _args = ('x', 'y')
 
-    x = NumberSpec(help="""
+    x = NumberSpec(default=field("x"), help="""
     The x-coordinates for the steps.
     """)
 
-    y = NumberSpec(help="""
+    y = NumberSpec(default=field("y"), help="""
     The y-coordinates for the steps.
     """)
 
@@ -1128,7 +1150,7 @@ class Step(XYGlyph):
     * ``center``: Draw step levels centered on each x-coordinate
     """)
 
-class Text(XYGlyph):
+class Text(XYGlyph, TextGlyph):
     ''' Render text.
 
     '''
@@ -1139,11 +1161,11 @@ class Text(XYGlyph):
     # functions derived from this class
     _args = ('x', 'y', 'text', 'angle', 'x_offset', 'y_offset')
 
-    x = NumberSpec(help="""
+    x = NumberSpec(default=field("x"), help="""
     The x-coordinates to locate the text anchors.
     """)
 
-    y = NumberSpec(help="""
+    y = NumberSpec(default=field("y"), help="""
     The y-coordinates to locate the text anchors.
     """)
 
@@ -1173,7 +1195,7 @@ class Text(XYGlyph):
     The %s values for the text.
     """)
 
-class VArea(Glyph):
+class VArea(FillGlyph, HatchGlyph):
     ''' Render a vertically directed area between two equal length sequences
     of y-coordinates with the same x-coordinates.
 
@@ -1185,27 +1207,27 @@ class VArea(Glyph):
     # functions derived from this class
     _args = ('x', 'y1', 'y2')
 
-    x = NumberSpec(help="""
+    x = NumberSpec(default=field("x"), help="""
     The x-coordinates for the points of the area.
     """)
 
-    y1 = NumberSpec(help="""
+    y1 = NumberSpec(default=field("y1"), help="""
     The y-coordinates for the points of one side of the area.
     """)
 
-    y2 = NumberSpec(help="""
+    y2 = NumberSpec(default=field("y2"), help="""
     The y-coordinates for the points of the other side of the area.
     """)
 
     fill_props = Include(ScalarFillProps, use_prefix=False, help="""
-    The %s values for the patch.
+    The %s values for the vertical directed area
     """)
 
     hatch_props = Include(HatchProps, use_prefix=False, help="""
     The %s values for the horizontal bars.
     """)
 
-class VBar(Glyph):
+class VBar(LineGlyph, FillGlyph, HatchGlyph):
     ''' Render vertical bars, given a center coordinate, width and (top, bottom) coordinates.
 
     '''
@@ -1216,7 +1238,7 @@ class VBar(Glyph):
     # functions derived from this class
     _args = ('x', 'width', 'top', 'bottom')
 
-    x = NumberSpec(help="""
+    x = NumberSpec(default=field("x"), help="""
     The x-coordinates of the centers of the vertical bars.
     """)
 
@@ -1244,7 +1266,7 @@ class VBar(Glyph):
     The %s values for the vertical bars.
     """)
 
-class Wedge(XYGlyph):
+class Wedge(XYGlyph, LineGlyph, FillGlyph):
     ''' Render wedges.
 
     '''
@@ -1255,11 +1277,11 @@ class Wedge(XYGlyph):
     # functions derived from this class
     _args = ('x', 'y', 'radius', 'start_angle', 'end_angle', 'direction')
 
-    x = NumberSpec(help="""
+    x = NumberSpec(default=field("x"), help="""
     The x-coordinates of the points of the wedges.
     """)
 
-    y = NumberSpec(help="""
+    y = NumberSpec(default=field("y"), help="""
     The y-coordinates of the points of the wedges.
     """)
 
@@ -1299,10 +1321,9 @@ class Wedge(XYGlyph):
 # Code
 #-----------------------------------------------------------------------------
 
-# XXX: allow `from bokeh.models.glyphs import *`
-from .markers import (Asterisk, Circle, CircleCross, CircleX, Cross, Dash,
-                      Diamond, DiamondCross, Hex, InvertedTriangle, Marker,
-                      Square, SquareCross, SquareX, Triangle, X)
+# XXX: allow `from bokeh.models.glyphs import *
+from .markers import (Asterisk, Circle, CircleCross, CircleX, Cross, Dash, Diamond, DiamondCross, # isort:skip
+                      Hex, InvertedTriangle, Marker, Square, SquareCross, SquareX, Triangle, X)   # isort:skip
 
 # Fool pyflakes
 (Asterisk, Circle, CircleCross, CircleX, Cross, Dash, Diamond, DiamondCross,
