@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -13,9 +13,7 @@ Mappers (as opposed to scales) are not presumed to be invertible.
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
@@ -25,13 +23,24 @@ log = logging.getLogger(__name__)
 # Standard library imports
 import warnings
 
-# External imports
-
 # Bokeh imports
 from .. import palettes
-from ..core.has_props import abstract
-from ..core.properties import Color, Either, Enum, Float, HatchPatternType, Int, MarkerType, Seq, String, Tuple
 from ..core.enums import Palette
+from ..core.has_props import abstract
+from ..core.properties import (
+    Color,
+    Either,
+    Enum,
+    Float,
+    HatchPatternType,
+    Instance,
+    Int,
+    List,
+    MarkerType,
+    Seq,
+    String,
+    Tuple,
+)
 from .transforms import Transform
 
 #-----------------------------------------------------------------------------
@@ -48,6 +57,7 @@ __all__ = (
     'ContinuousColorMapper',
     'LinearColorMapper',
     'LogColorMapper',
+    'EqHistColorMapper'
 )
 
 #-----------------------------------------------------------------------------
@@ -81,7 +91,7 @@ class ColorMapper(Mapper):
     def __init__(self, palette=None, **kwargs):
         if palette is not None:
             kwargs['palette'] = palette
-        super(ColorMapper, self).__init__(**kwargs)
+        super().__init__(**kwargs)
 
 @abstract
 class CategoricalMapper(Mapper):
@@ -131,7 +141,7 @@ class CategoricalColorMapper(CategoricalMapper, ColorMapper):
     '''
 
     def __init__(self, **kwargs):
-        super(CategoricalColorMapper, self).__init__(**kwargs)
+        super().__init__(**kwargs)
         palette = self.palette
         factors = self.factors
         if palette is not None and factors is not None:
@@ -185,14 +195,19 @@ class ContinuousColorMapper(ColorMapper):
 
     '''
 
-    low = Float(help="""
-    The minimum value of the range to map into the palette. Values below
-    this are clamped to ``low``.
+    domain = List(Tuple(Instance("bokeh.models.renderers.GlyphRenderer"), Either(String, List(String))), default=[], help="""
+    A collection of glyph renderers to pool data from for establishing data metrics.
+    If empty, mapped data will be used instead.
     """)
 
-    high = Float(help="""
+    low = Float(default=None, help="""
+    The minimum value of the range to map into the palette. Values below
+    this are clamped to ``low``. If ``None``, the value is inferred from data.
+    """)
+
+    high = Float(default=None, help="""
     The maximum value of the range to map into the palette. Values above
-    this are clamped to ``high``.
+    this are clamped to ``high``. If ``None``, the value is inferred from data.
     """)
 
     low_color = Color(default=None, help="""
@@ -239,6 +254,10 @@ class LogColorMapper(ContinuousColorMapper):
 
     '''
 
+class EqHistColorMapper(ContinuousColorMapper):
+    bins = Int(default=256*256, help="Number of histogram bins")
+
+
 #-----------------------------------------------------------------------------
 # Dev API
 #-----------------------------------------------------------------------------
