@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -12,18 +12,29 @@ The full list of markers built into Bokeh is given below:
 * :class:`~bokeh.models.markers.Asterisk`
 * :class:`~bokeh.models.markers.Circle`
 * :class:`~bokeh.models.markers.CircleCross`
+* :class:`~bokeh.models.markers.CircleDot`
+* :class:`~bokeh.models.markers.CircleY`
 * :class:`~bokeh.models.markers.CircleX`
 * :class:`~bokeh.models.markers.Cross`
 * :class:`~bokeh.models.markers.Dash`
 * :class:`~bokeh.models.markers.Diamond`
 * :class:`~bokeh.models.markers.DiamondCross`
+* :class:`~bokeh.models.markers.DiamondDot`
+* :class:`~bokeh.models.markers.Dot`
 * :class:`~bokeh.models.markers.Hex`
+* :class:`~bokeh.models.markers.HexDot`
 * :class:`~bokeh.models.markers.InvertedTriangle`
+* :class:`~bokeh.models.markers.Plus`
 * :class:`~bokeh.models.markers.Square`
 * :class:`~bokeh.models.markers.SquareCross`
+* :class:`~bokeh.models.markers.SquareDot`
+* :class:`~bokeh.models.markers.SquarePin`
 * :class:`~bokeh.models.markers.SquareX`
 * :class:`~bokeh.models.markers.Triangle`
+* :class:`~bokeh.models.markers.TriangleDot`
+* :class:`~bokeh.models.markers.TrianglePin`
 * :class:`~bokeh.models.markers.X`
+* :class:`~bokeh.models.markers.Y`
 
 Markers are all subclasses of ``Glyph``. Additionally, they all share the
 same common interface providing fill and line properties provided by their
@@ -40,26 +51,28 @@ other markers do not.
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
 # Imports
 #-----------------------------------------------------------------------------
 
-# Standard library imports
-
-# External imports
-
 # Bokeh imports
 from ..core.enums import enumeration
 from ..core.has_props import abstract
-from ..core.properties import AngleSpec, DistanceSpec, Enum, Include, MarkerSpec, NumberSpec, ScreenDistanceSpec
+from ..core.properties import (
+    AngleSpec,
+    DistanceSpec,
+    Enum,
+    Include,
+    MarkerSpec,
+    NumberSpec,
+    ScreenDistanceSpec,
+)
+from ..core.property.dataspec import field
 from ..core.property_mixins import FillProps, LineProps
-
-from .glyphs import XYGlyph
+from .glyph import FillGlyph, LineGlyph, XYGlyph
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -69,20 +82,31 @@ __all__ = (
     'Asterisk',
     'Circle',
     'CircleCross',
+    'CircleDot',
     'CircleX',
+    'CircleY',
     'Cross',
     'Dash',
     'Diamond',
     'DiamondCross',
+    'DiamondDot',
+    'Dot',
     'Hex',
+    'HexDot',
     'InvertedTriangle',
     'Marker',
+    'Plus',
     'Scatter',
     'Square',
     'SquareCross',
+    'SquareDot',
+    'SquarePin',
     'SquareX',
     'Triangle',
+    'TriangleDot',
+    'TrianglePin',
     'X',
+    'Y',
 )
 
 #-----------------------------------------------------------------------------
@@ -90,7 +114,7 @@ __all__ = (
 #-----------------------------------------------------------------------------
 
 @abstract
-class Marker(XYGlyph):
+class Marker(XYGlyph, LineGlyph, FillGlyph):
     ''' Base class for glyphs that are simple markers with line and
     fill properties, located at an (x, y) location with a specified
     size.
@@ -107,11 +131,11 @@ class Marker(XYGlyph):
     # functions derived from this class
     _args = ('x', 'y', 'size', 'angle')
 
-    x = NumberSpec(help="""
+    x = NumberSpec(default=field("x"), help="""
     The x-axis coordinates for the center of the markers.
     """)
 
-    y = NumberSpec(help="""
+    y = NumberSpec(default=field("y"), help="""
     The y-axis coordinates for the center of the markers.
     """)
 
@@ -221,11 +245,21 @@ class CircleCross(Marker):
 
     __example__ = "examples/reference/models/CircleCross.py"
 
+class CircleDot(Marker):
+    ''' Render circle markers with center dots. '''
+
+    __example__ = "examples/reference/models/CircleDot.py"
+
 class CircleX(Marker):
     ''' Render circle markers with an 'X' cross through the center. '''
 
     __example__ = "examples/reference/models/CircleX.py"
 
+class CircleY(Marker):
+    ''' Render circle markers with an 'Y' cross through the center. '''
+
+    __example__ = "examples/reference/models/CircleY.py"
+
 class Cross(Marker):
     ''' Render '+' cross markers. '''
 
@@ -248,21 +282,51 @@ class DiamondCross(Marker):
 
     __example__ = "examples/reference/models/DiamondCross.py"
 
+class DiamondDot(Marker):
+    ''' Render diamond markers with center dots. '''
+
+    __example__ = "examples/reference/models/DiamondDot.py"
+
+class Dot(Marker):
+    ''' Render dots (one-quarter radius circles). '''
+
+    __example__ = "examples/reference/models/Dot.py"
+
 class Hex(Marker):
     ''' Render hexagon markers. '''
 
     __example__ = "examples/reference/models/Hex.py"
 
+class HexDot(Marker):
+    ''' Render hexagon markers with center dots. '''
+
+    __example__ = "examples/reference/models/HexDot.py"
+
 class InvertedTriangle(Marker):
     ''' Render upside-down triangle markers. '''
 
     __example__ = "examples/reference/models/InvertedTriangle.py"
 
+class Plus(Marker):
+    ''' Render filled plus markers '''
+
+    __example__ = "examples/reference/models/Plus.py"
+
 class Square(Marker):
-    ''' Render a square marker, optionally rotated. '''
+    ''' Render square markers. '''
 
     __example__ = "examples/reference/models/Square.py"
 
+class SquareDot(Marker):
+    ''' Render square markers with center dots. '''
+
+    __example__ = "examples/reference/models/SquareDot.py"
+
+class SquarePin(Marker):
+    ''' Render pin-cushion square markers. '''
+
+    __example__ = "examples/reference/models/SquarePin.py"
+
 class SquareCross(Marker):
     ''' Render square markers with a '+' cross through the center. '''
 
@@ -278,15 +342,59 @@ class Triangle(Marker):
 
     __example__ = "examples/reference/models/Triangle.py"
 
+class TriangleDot(Marker):
+    ''' Render triangle markers with center dots. '''
+
+    __example__ = "examples/reference/models/TriangleDot.py"
+
+class TrianglePin(Marker):
+    ''' Render pin-cushion triangle markers. '''
+
+    __example__ = "examples/reference/models/TrianglePin.py"
+
 class X(Marker):
-    ''' Render a 'X' cross markers. '''
+    ''' Render 'X' markers. '''
 
     __example__ = "examples/reference/models/X.py"
 
+class Y(Marker):
+    ''' Render 'Y' markers. '''
+
+    __example__ = "examples/reference/models/Y.py"
+
 #-----------------------------------------------------------------------------
 # Dev API
 #-----------------------------------------------------------------------------
 
+marker_types = {
+    "asterisk": Asterisk,
+    "circle": Circle,
+    "circle_cross": CircleCross,
+    "circle_dot": CircleDot,
+    "circle_x": CircleX,
+    "circle_y": CircleY,
+    "cross": Cross,
+    "dash": Dash,
+    "diamond": Diamond,
+    "diamond_cross": DiamondCross,
+    "diamond_dot": DiamondDot,
+    "dot": Dot,
+    "hex": Hex,
+    "hex_dot": HexDot,
+    "inverted_triangle": InvertedTriangle,
+    "plus": Plus,
+    "square": Square,
+    "square_cross": SquareCross,
+    "square_dot": SquareDot,
+    "square_pin": SquarePin,
+    "square_x": SquareX,
+    "triangle": Triangle,
+    "triangle_dot": TriangleDot,
+    "triangle_pin": TrianglePin,
+    "x": X,
+    "y": Y,
+}
+
 #-----------------------------------------------------------------------------
 # Private API
 #-----------------------------------------------------------------------------
