@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -13,9 +13,7 @@ and with options for "auto sizing".
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
@@ -25,18 +23,26 @@ log = logging.getLogger(__name__)
 # Standard library imports
 from collections import Counter
 
-# External imports
-
 # Bokeh imports
 from ..core.enums import PaddingUnits, StartEnd
 from ..core.has_props import abstract
-from ..core.properties import (Bool, Datetime, Either, Enum, Float, Instance,
-                               List, MinMaxBounds, Seq, String, TimeDelta, Tuple)
+from ..core.properties import (
+    Bool,
+    Datetime,
+    Either,
+    Enum,
+    Float,
+    Instance,
+    List,
+    MinMaxBounds,
+    Seq,
+    String,
+    TimeDelta,
+    Tuple,
+)
 from ..core.validation import error
 from ..core.validation.errors import DUPLICATE_FACTORS
 from ..model import Model
-
-from .callbacks import Callback
 from .renderers import Renderer
 
 #-----------------------------------------------------------------------------
@@ -61,11 +67,6 @@ class Range(Model):
 
     '''
 
-    callback = Instance(Callback, help="""
-    A callback to run in the browser whenever the range is updated.
-    """)
-
-
 class Range1d(Range):
     ''' A fixed, closed range [start, end] in a continuous scalar
     dimension.
@@ -138,7 +139,7 @@ class Range1d(Range):
             kwargs['start'] = args[0]
             kwargs['end'] = args[1]
 
-        super(Range1d, self).__init__(**kwargs)
+        super().__init__(**kwargs)
 
 
 @abstract
@@ -266,7 +267,7 @@ class DataRange1d(DataRange):
     def __init__(self, *args, **kwargs):
         if kwargs.get('follow') is not None:
             kwargs['bounds'] = None
-        super(DataRange1d, self).__init__(**kwargs)
+        super().__init__(**kwargs)
 
 
 class FactorRange(Range):
@@ -394,7 +395,7 @@ class FactorRange(Range):
     The start of the range, in synthetic coordinates.
 
         Synthetic coordinates are only computed in the browser, based on the
-        factors and various padding properties. The value of ``end`` will only
+        factors and various padding properties. The value of ``start`` will only
         be available in situations where bidirectional communication is
         available (e.g. server, notebook).
     """)
@@ -449,7 +450,7 @@ class FactorRange(Range):
             raise ValueError("'factors' keyword cannot be used with positional arguments")
         elif args:
             kwargs['factors'] = list(args)
-        super(FactorRange, self).__init__(**kwargs)
+        super().__init__(**kwargs)
 
     @error(DUPLICATE_FACTORS)
     def _check_duplicate_factors(self):
