@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -8,9 +8,7 @@
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
@@ -20,17 +18,28 @@ log = logging.getLogger(__name__)
 # Standard library imports
 import warnings
 
-# External imports
-
 # Bokeh imports
 from ..core.has_props import abstract
-from ..core.properties import Any, Bool, ColumnData, Dict, Enum, Instance, Int, JSON, List, PandasDataFrame, PandasGroupBy, Seq, String
+from ..core.properties import (
+    JSON,
+    Any,
+    Bool,
+    ColumnData,
+    Dict,
+    Enum,
+    Instance,
+    Int,
+    List,
+    PandasDataFrame,
+    PandasGroupBy,
+    Seq,
+    String,
+)
 from ..model import Model
 from ..util.dependencies import import_optional
 from ..util.serialization import convert_datetime_array
 from ..util.warnings import BokehUserWarning
-
-from .callbacks import Callback, CustomJS
+from .callbacks import CustomJS
 from .filters import Filter
 from .selections import Selection, SelectionPolicy, UnionRenderers
 
@@ -47,7 +56,6 @@ __all__ = (
     'ColumnDataSource',
     'DataSource',
     'GeoJSONDataSource',
-    'RemoteSource',
     'ServerSentDataSource',
     'WebSource',
 )
@@ -62,16 +70,10 @@ class DataSource(Model):
 
     '''
 
-    selected = Instance(Selection, default=lambda: Selection(), help="""
-    A Selection that indicates selected indices on this ``DataSource``.
-    """)
-
-    callback = Instance(Callback, help="""
-    A callback to run in the browser whenever the selection is changed.
-
-    .. note:
-        This property is left for backwards compatibility, but may be deprecated
-        in the future. Prefer ``source.selected.js_on_change(...)`` for new code.
+    selected = Instance(Selection, default=lambda: Selection(), readonly=True, help="""
+    An instance of a ``Selection`` that indicates selected indices on this ``DataSource``.
+    This is a read-only property. You may only change the attributes of this object
+    to change the selection (e.g., ``selected.indices``).
     """)
 
 @abstract
@@ -167,7 +169,7 @@ class ColumnDataSource(ColumnarDataSource):
         PandasDataFrame, lambda x: ColumnDataSource._data_from_df(x)
     ).accepts(
         PandasGroupBy, lambda x: ColumnDataSource._data_from_groupby(x)
-    ).asserts(lambda _, data: len(set(len(x) for x in data.values())) <= 1,
+    ).asserts(lambda _, data: len({len(x) for x in data.values()}) <= 1,
                  lambda obj, name, data: warnings.warn(
                     "ColumnDataSource's columns must be of the same length. " +
                     "Current lengths: %s" % ", ".join(sorted(str((k, len(v))) for k, v in data.items())), BokehUserWarning))
@@ -190,7 +192,7 @@ class ColumnDataSource(ColumnarDataSource):
                 raw_data = self._data_from_groupby(raw_data)
             else:
                 raise ValueError("expected a dict or pandas.DataFrame, got %s" % raw_data)
-        super(ColumnDataSource, self).__init__(**kw)
+        super().__init__(**kw)
         self.data.update(raw_data)
 
     @property
@@ -232,7 +234,7 @@ class ColumnDataSource(ColumnarDataSource):
             _df.index = pd.Index(_df.index.values, name=index_name)
         _df.reset_index(inplace=True)
 
-        tmp_data = {c: v.values for c, v in _df.iteritems()}
+        tmp_data = {c: v.values for c, v in _df.items()}
 
         new_data = {}
         for k, v in tmp_data.items():
@@ -368,7 +370,6 @@ class ColumnDataSource(ColumnarDataSource):
         try:
             del self.data[name]
         except (ValueError, KeyError):
-            import warnings
             warnings.warn("Unable to find column '%s' in data source" % name)
 
     def stream(self, new_data, rollover=None):
@@ -475,7 +476,7 @@ class ColumnDataSource(ColumnarDataSource):
             newkeys = set(_df.columns)
             index_name = ColumnDataSource._df_index_name(_df)
             newkeys.add(index_name)
-            new_data = dict(_df.iteritems())
+            new_data = dict(_df.items())
             new_data[index_name] = _df.index.values
         else:
             newkeys = set(new_data.keys())
@@ -743,27 +744,13 @@ class WebSource(ColumnDataSource):
     A URL to to fetch data from.
     """)
 
-@abstract
-class RemoteSource(WebSource):
-    ''' Base class for remote column data sources that can update from data
-    URLs at prescribed time intervals.
-
-    .. note::
-        This base class is typically not useful to instantiate on its own.
-
-    '''
-
-    polling_interval = Int(help="""
-    A polling interval (in milliseconds) for updating data source.
-    """)
-
 class ServerSentDataSource(WebSource):
     ''' A data source that can populate columns by receiving server sent
     events endpoints.
 
     '''
 
-class AjaxDataSource(RemoteSource):
+class AjaxDataSource(WebSource):
     ''' A data source that can populate columns by making Ajax calls to REST
     endpoints.
 
@@ -794,6 +781,10 @@ class AjaxDataSource(RemoteSource):
 
     '''
 
+    polling_interval = Int(help="""
+    A polling interval (in milliseconds) for updating data source.
+    """)
+
     method = Enum('POST', 'GET', help="""
     Specify the HTTP method to use for the Ajax request (GET or POST)
     """)
