@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -11,25 +11,34 @@
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
 # Imports
 #-----------------------------------------------------------------------------
 
-# Standard library imports
-
-# External imports
-
 # Bokeh imports
+from ...core.enums import CalendarPosition
 from ...core.has_props import abstract
-from ...core.properties import ColorHex, Date, Dict, Either, Float, Instance, Int, List, PositiveInt, String, Tuple
-
-from ..callbacks import Callback
-
+from ...core.properties import (
+    Bool,
+    ColorHex,
+    Date,
+    Dict,
+    Either,
+    Enum,
+    Float,
+    Instance,
+    Int,
+    Interval,
+    List,
+    Override,
+    PositiveInt,
+    String,
+    Tuple,
+)
+from ..formatters import TickFormatter
 from .widget import Widget
 
 #-----------------------------------------------------------------------------
@@ -42,7 +51,9 @@ __all__ = (
     'DatePicker',
     'FileInput',
     'InputWidget',
+    'MultiChoice',
     'MultiSelect',
+    'NumericInput',
     'PasswordInput',
     'Select',
     'Spinner',
@@ -82,23 +93,49 @@ class InputWidget(Widget):
 #-----------------------------------------------------------------------------
 
 class FileInput(Widget):
-    ''' Present a file-chooser dialog to users and return the contents of a
-    selected file.
-
+    ''' Present a file-chooser dialog to users and return the contents of the
+    selected files.
     '''
 
-    value = String(default="", readonly=True, help="""
-    A base64-encoded string of the contents of the selected file.
-    """)
+    value = Either(String, List(String), default='', readonly=True, help='''
+    The base64-enconded contents of the file or files that were loaded.
 
-    mime_type = String(default="", readonly=True, help="""
-    The mime type of the selected file.
-    """)
+    If `mulitiple` is set to False (default), this value is a single string with the contents
+    of the single file that was chosen.
 
-    filename = String(default="", readonly=True, help="""
-    The filename of the selected file.
-    The file path is not included as browsers do not allow access to it.
-    """)
+    If `multiple` is True, this value is a list of strings, each containing the contents of
+    one of the multiple files that were chosen.
+
+    The sequence of files is given by the list of filenames (see below)
+    ''')
+
+    mime_type = Either(String, List(String), default='', readonly=True, help='''
+    The mime-type of the file or files that were loaded.
+
+    If `mulitiple` is set to False (default), this value is a single string with the
+    mime-type of the single file that was chosen.
+
+    If `multiple` is True, this value is a list of strings, each containing the
+    mime-type of one of the multiple files that were chosen.
+
+    The sequence of files is given by the list of filename (see below)
+    ''')
+
+    filename = Either(String, List(String), default='', readonly=True, help='''
+    The name(s) of the file or files that were loaded.
+
+    If `mulitiple` is set to False (default), this value is a single string with the
+    name of the single file that was chosen.
+
+    If `multiple` is True, this value is a list of strings, each containing the
+    name of one of the multiple files that were chosen.
+
+    This list provides the sequence of files for the respective lists in value and mime-type
+
+    .. note::
+        The full file path is not included since browsers will not provide
+        access to that information for security reasons.
+    ''')
 
     accept = String(default="", help="""
     Comma-separated list of standard HTML file input filters that restrict what
@@ -122,6 +159,72 @@ class FileInput(Widget):
     .. _IANA Media Type: https://www.iana.org/assignments/media-types/media-types.xhtml
     """)
 
+    multiple = Bool(default=False, help="""
+    set multiple=False (default) for single file selection, set multiple=True if
+    selection of more than one file at a time should be possible.
+    """)
+
+
+class NumericInput(InputWidget):
+    ''' Numeric input widget.
+
+    '''
+
+    value = Either(Float, Int, help="""
+    Initial or entered value.
+
+    Change events are triggered whenever <enter> is pressed.
+    """)
+
+    low = Either(Float, Int, help="""
+    Optional lowest allowable value.
+    """)
+
+    high = Either(Float, Int, help="""
+    Optional highest allowable value.
+    """)
+
+    placeholder = String(default="", help="""
+    Placeholder for empty input field.
+    """)
+
+    mode = Enum("int", "float", help="""
+    Define the type of number which can be enter in the input
+
+    example
+    mode int: 1, -1, 156
+    mode float: 1, -1.2, 1.1e-25
+    """)
+
+    format = Either(String, Instance(TickFormatter), help="""
+    """)
+
+
+class Spinner(NumericInput):
+    ''' Numeric Spinner input widget.
+
+    '''
+
+    value_throttled = Either(Float, Int, help="""
+    value reported at the end of interactions
+    """)
+
+    mode = Override(default="float")
+
+    step = Interval(Float, start=1e-16, end=float('inf'), default=1, help="""
+    The step added or subtracted to the current value
+    """)
+
+    page_step_multiplier = Interval(Float, start=0, end=float('inf'), default=10, help="""
+    Defines the multiplication factor applied to step when the page up and page
+    down keys are pressed
+    """)
+
+    wheel_wait = Either(Int, Float, default=100, help="""
+    Defines the debounce time in ms before updating `value_throttled` when the
+    mouse wheel is used to change the input
+    """)
+
 
 class TextInput(InputWidget):
     ''' Single-line input widget.
@@ -130,18 +233,15 @@ class TextInput(InputWidget):
 
     value = String(default="", help="""
     Initial or entered text value.
+
+    Change events are triggered whenever <enter> is pressed.
     """)
 
     value_input = String(default="", help="""
-    Initial or entered text value that triggers a callback whenever the value changes.
-    """)
+    Initial or current value.
 
-    callback = Instance(Callback, help="""
-    A callback to run in the browser whenever the user unfocuses the
-    ``TextInput`` widget by hitting Enter or clicking outside of the text box
-    area.
-
-    DEPRECATED: use .js_on_change or .on_change with "value" or "value_input"
+    Change events are triggered whenever any update happens, i.e. on every
+    keypress.
     """)
 
     placeholder = String(default="", help="""
@@ -169,11 +269,12 @@ class TextAreaInput(TextInput):
 
 class PasswordInput(TextInput):
     ''' Single-line password input widget.
-        Note: Despite ``PasswordInput`` inheriting from ``TextInput`` the
-        password cannot be inspected on the field ``value``. Also, note that
-        this field functionally just hides the input on the browser,
-        transmitting safely a password as a callback, e.g., to a bokeh
-        server would require some secure connection.
+
+    This widget hides the input value so that it is not visible in the browser.
+
+    .. warning::
+        Secure transmission of the password to Bokeh server application code
+        requires configuring the server for SSL (i.e. HTTPS) termination.
 
     '''
 
@@ -192,13 +293,15 @@ class AutocompleteInput(TextInput):
     The number of characters a user must type before completions are presented.
     """)
 
+    case_sensitive = Bool(default=True, help="""Enable or disable case sensitivity""")
+
 
 class Select(InputWidget):
     ''' Single-select widget.
 
     '''
-    options = Either(List(Either(String, Tuple(Either(Int, String), String))),
-        Dict(String, List(Either(String, Tuple(Either(Int, String), String)))), help="""
+    options = Either(List(Either(String, Tuple(String, String))),
+        Dict(String, List(Either(String, Tuple(String, String)))), help="""
     Available selection options. Options may be provided either as a list of
     possible string values, or as a list of tuples, each of the form
     ``(value, label)``. In the latter case, the visible widget text for each
@@ -211,12 +314,6 @@ class Select(InputWidget):
     Initial or selected value.
     """)
 
-    callback = Instance(Callback, help="""
-    A callback to run in the browser whenever the current Select dropdown
-    value changes.
-    """)
-
-
 class MultiSelect(InputWidget):
     ''' Multi-select widget.
 
@@ -233,11 +330,6 @@ class MultiSelect(InputWidget):
     Initial or selected values.
     """)
 
-    callback = Instance(Callback, help="""
-    A callback to run in the browser whenever the current selection value
-    changes.
-    """)
-
     size = Int(default=4, help="""
     The number of visible options in the dropdown list. (This uses the
     ``select`` HTML element's ``size`` attribute. Some browsers might not
@@ -245,6 +337,42 @@ class MultiSelect(InputWidget):
     """)
 
 
+class MultiChoice(InputWidget):
+    ''' MultiChoice widget.
+
+    '''
+
+    options = List(Either(String, Tuple(String, String)), help="""
+    Available selection options. Options may be provided either as a list of
+    possible string values, or as a list of tuples, each of the form
+    ``(value, label)``. In the latter case, the visible widget text for each
+    value will be corresponding given label.
+    """)
+
+    value = List(String, help="""
+    Initial or selected values.
+    """)
+
+    delete_button = Bool(default=True, help="""
+    Whether to add a button to remove a selected option.
+    """)
+
+    max_items = Int(default=None, help="""
+    The maximum number of items that can be selected.
+    """)
+
+    option_limit = Int(default=None, help="""
+    The number of choices that will be rendered in the dropdown.
+    """)
+
+    placeholder = String(default=None, help="""
+    A string that is displayed if not item is added.
+    """)
+
+    solid = Bool(default=True, help="""
+    Specify whether the choices should be solidly filled.""")
+
+
 class DatePicker(InputWidget):
     ''' Calendar-based date picker widget.
 
@@ -262,10 +390,29 @@ class DatePicker(InputWidget):
     Optional latest allowable date.
     """)
 
-    callback = Instance(Callback, help="""
-    A callback to run in the browser whenever the current date value changes.
+    disabled_dates = List(Either(Date, Tuple(Date, Date)), default=[], help="""
+    A list of dates of ``(start, end)`` date ranges to make unavailable for
+    selection. All other dates will be avalable.
+
+    .. note::
+        Only one of ``disabled_dates`` and ``enabled_dates`` should be specified.
     """)
 
+    enabled_dates = List(Either(Date, Tuple(Date, Date)), default=[], help="""
+    A list of dates of ``(start, end)`` date ranges to make available for
+    selection. All other dates will be unavailable.
+
+    .. note::
+        Only one of ``disabled_dates`` and ``enabled_dates`` should be specified.
+    """)
+
+    position = Enum(CalendarPosition, default="auto", help="""
+    Where the calendar is rendered relative to the input when ``inline`` is False.
+    """)
+
+    inline = Bool(default=False, help="""
+    Whether the calendar sholud be displayed inline.
+    """)
 
 class ColorPicker(InputWidget):
     ''' Color picker widget
@@ -280,38 +427,6 @@ class ColorPicker(InputWidget):
     The initial color of the picked color (named or hexadecimal)
     """)
 
-    callback = Instance(Callback, help="""
-    A callback to run in the browser whenever the current date value changes.
-    """)
-
-
-class Spinner(InputWidget):
-    ''' Spinner widget for numerical inputs
-
-    '''
-
-    value = Float(default=0, help="""
-    The initial value of the spinner
-    """)
-
-    step = Float(default=1, help="""
-    The step added or subtracted to the current value
-    """)
-
-    low = Float(help="""
-    Optional lowest allowable value.
-    """)
-
-    high = Float(help="""
-    Optional highest allowable value.
-    """)
-
-    callback = Instance(Callback, help="""
-    A callback to run in the browser whenever the user unfocuses the
-    ``SpinBox`` widget by hitting Enter or clicking outside of the box
-    area.
-    """)
-
 #-----------------------------------------------------------------------------
 # Private API
 #-----------------------------------------------------------------------------
