@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -12,24 +12,28 @@ Servers and clients.
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
 # Imports
 #-----------------------------------------------------------------------------
 
-# Standard library imports
-
 # External imports
 from tornado.escape import json_decode
 
 # Bokeh imports
 from . import messages
-from . import versions
 from .exceptions import ProtocolError
+from .messages.ack import ack
+from .messages.error import error
+from .messages.ok import ok
+from .messages.patch_doc import patch_doc
+from .messages.pull_doc_reply import pull_doc_reply
+from .messages.pull_doc_req import pull_doc_req
+from .messages.push_doc import push_doc
+from .messages.server_info_reply import server_info_reply
+from .messages.server_info_req import server_info_req
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -39,6 +43,18 @@ __all__ = (
     'Protocol',
 )
 
+SPEC = {
+    "ACK": ack,
+    "ERROR": error,
+    "OK": ok,
+    'PATCH-DOC': patch_doc,
+    'PULL-DOC-REPLY': pull_doc_reply,
+    'PULL-DOC-REQ': pull_doc_req,
+    'PUSH-DOC': push_doc,
+    'SERVER-INFO-REPLY': server_info_reply,
+    'SERVER-INFO-REQ': server_info_req,
+}
+
 #-----------------------------------------------------------------------------
 # General API
 #-----------------------------------------------------------------------------
@@ -47,26 +63,15 @@ __all__ = (
 # Dev API
 #-----------------------------------------------------------------------------
 
-class Protocol(object):
-    ''' Provide a message factory for a given version of the Bokeh Server
-    message protocol.
-
-    Args:
-        version (str) : a string identifying a protocol version, e.g. "1.0"
+class Protocol:
+    ''' Provide a message factory for the Bokeh Server message protocol.
 
     '''
-    def __init__(self, version):
-        if version not in versions.spec:
-            raise ProtocolError("Unknown protocol version %r" % version)
-
-        self._version = version
-        self._messages = dict()
-
-        for msgtype, revision in versions.spec[version]:
-            self._messages[msgtype] = messages.index[(msgtype, revision)]
+    def __init__(self):
+        self._messages = SPEC
 
     def __repr__(self):
-        return "Protocol(%r)" % self.version
+        return "Protocol()"
 
     def create(self, msgtype, *args, **kwargs):
         ''' Create a new Message instance for the given type.
@@ -76,7 +81,7 @@ class Protocol(object):
 
         '''
         if msgtype not in self._messages:
-            raise ProtocolError("Unknown message type %r for protocol version %s" % (msgtype, self._version))
+            raise ProtocolError("Unknown message type %r for Bokeh protocol" % msgtype)
         return self._messages[msgtype].create(*args, **kwargs)
 
     def assemble(self, header_json, metadata_json, content_json):
@@ -101,10 +106,6 @@ class Protocol(object):
             header_json, metadata_json, content_json
         )
 
-    @property
-    def version(self):
-        return self._version
-
 #-----------------------------------------------------------------------------
 # Private API
 #-----------------------------------------------------------------------------
