@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -8,9 +8,7 @@
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
@@ -21,18 +19,15 @@ log = logging.getLogger(__name__)
 import sys
 from traceback import format_exception
 
-# External imports
-
 # Bokeh imports
 from ..message import Message
-from . import register
 
 #-----------------------------------------------------------------------------
 # Globals and constants
 #-----------------------------------------------------------------------------
 
 __all__ = (
-    'error_1',
+    'error',
 )
 
 #-----------------------------------------------------------------------------
@@ -43,10 +38,9 @@ __all__ = (
 # Dev API
 #-----------------------------------------------------------------------------
 
-@register
-class error_1(Message):
-    ''' Define the ``ERROR`` message (revision 1) for reporting error
-    conditions back to a Bokeh server.
+class error(Message):
+    ''' Define the ``ERROR`` message for reporting error conditions back to a
+    Bokeh server.
 
     The ``content`` fragment of for this message is has the form:
 
@@ -62,10 +56,9 @@ class error_1(Message):
     '''
 
     msgtype  = 'ERROR'
-    revision = 1
 
     def __repr__(self):
-        msg = super(error_1, self).__repr__()
+        msg = super().__repr__()
         msg += " --- "
         msg += self.content['text']
         if "traceback" in self.content:
