@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -11,9 +11,7 @@
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
@@ -21,14 +19,14 @@ log = logging.getLogger(__name__)
 #-----------------------------------------------------------------------------
 
 # Standard library imports
+import inspect
 import time
 
 # External imports
-from tornado import gen, locks
+from tornado import locks
 
 # Bokeh imports
-from ..util.tornado import yield_for_all_futures
-
+from ..util.token import generate_jwt_token
 from .callbacks import _DocumentCallbackGroup
 
 #-----------------------------------------------------------------------------
@@ -50,24 +48,27 @@ def _needs_document_lock(func):
        method on ServerSession and transforms it into a coroutine
        if it wasn't already.
     '''
-    @gen.coroutine
-    def _needs_document_lock_wrapper(self, *args, **kwargs):
+    async def _needs_document_lock_wrapper(self, *args, **kwargs):
         # while we wait for and hold the lock, prevent the session
         # from being discarded. This avoids potential weirdness
         # with the session vanishing in the middle of some async
         # task.
         if self.destroyed:
             log.debug("Ignoring locked callback on already-destroyed session.")
-            raise gen.Return(None)
+            return None
         self.block_expiration()
         try:
-            with (yield self._lock.acquire()):
+            with await self._lock.acquire():
                 if self._pending_writes is not None:
                     raise RuntimeError("internal class invariant violated: _pending_writes " + \
                                        "should be None if lock is not held")
                 self._pending_writes = []
                 try:
-                    result = yield yield_for_all_futures(func(self, *args, **kwargs))
+                    result = func(self, *args, **kwargs)
+                    if inspect.isawaitable(result):
+                        # Note that this must not be outside of the critical section.
+                        # Otherwise, the async callback will be ran without document locking.
+                        result = await result
                 finally:
                     # we want to be very sure we reset this or we'll
                     # keep hitting the RuntimeError above as soon as
@@ -75,8 +76,8 @@ def _needs_document_lock(func):
                     pending_writes = self._pending_writes
                     self._pending_writes = None
                 for p in pending_writes:
-                    yield p
-            raise gen.Return(result)
+                    await p
+            return result
         finally:
             self.unblock_expiration()
     return _needs_document_lock_wrapper
@@ -89,24 +90,20 @@ def current_time():
     '''Return the time in milliseconds since the epoch as a floating
        point number.
     '''
-    try:
-        # python >=3.3 only
-        return time.monotonic() * 1000
-    except:
-        # if your python is old, don't set your clock backward!
-        return time.time() * 1000
+    return time.monotonic() * 1000
 
-class ServerSession(object):
+class ServerSession:
     ''' Hosts an application "instance" (an instantiated Document) for one or more connections.
 
     '''
 
-    def __init__(self, session_id, document, io_loop=None):
+    def __init__(self, session_id, document, io_loop=None, token=None):
         if session_id is None:
             raise ValueError("Sessions must have an id")
         if document is None:
             raise ValueError("Sessions must have a document")
         self._id = session_id
+        self._token = token
         self._document = document
         self._loop = io_loop
         self._subscribed_connections = set()
@@ -131,6 +128,13 @@ class ServerSession(object):
     def id(self):
         return self._id
 
+    @property
+    def token(self):
+        ''' A JWT token to authenticate the session. '''
+        if self._token:
+            return self._token
+        return generate_jwt_token(self.id)
+
     @property
     def destroyed(self):
         return self._destroyed
@@ -260,16 +264,6 @@ class ServerSession(object):
 
         return connection.ok(message)
 
-    @_needs_document_lock
-    def _handle_event(self, message, connection):
-        message.notify_event(self.document)
-        return connection.ok(message)
-
-    @classmethod
-    def event(cls, message, connection):
-        return connection.session._handle_event(message, connection)
-
-
     @classmethod
     def patch(cls, message, connection):
         ''' Handle a PATCH-DOC, return a Future with work to be scheduled. '''
