@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -11,9 +11,7 @@
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
@@ -22,10 +20,11 @@ log = logging.getLogger(__name__)
 
 # Standard library imports
 import os
+import sys
 from pprint import pformat
+from urllib.parse import urljoin
 
 # External imports
-from tornado import gen
 from tornado.ioloop import PeriodicCallback
 from tornado.web import Application as TornadoApplication
 from tornado.web import StaticFileHandler
@@ -35,10 +34,10 @@ from ..application import Application
 from ..resources import Resources
 from ..settings import settings
 from ..util.dependencies import import_optional
-
+from ..util.string import format_docstring
 from .auth_provider import NullAuth
-from .contexts import ApplicationContext
 from .connection import ServerConnection
+from .contexts import ApplicationContext
 from .urls import per_app_patterns, toplevel_patterns
 from .views.root_handler import RootHandler
 from .views.static_handler import StaticHandler
@@ -47,15 +46,13 @@ from .views.static_handler import StaticHandler
 # Globals and constants
 #-----------------------------------------------------------------------------
 
-# Unfortunately we can't yet use format_docstring to keep these automatically in sync in
-# the class docstring because Python 2 does not allow setting class.__doc__ (works with
-# Bokeh model classes because they are metaclasses)
 DEFAULT_CHECK_UNUSED_MS                  = 17000
 DEFAULT_KEEP_ALIVE_MS                    = 37000  # heroku, nginx default to 60s timeout, so use less than that
 DEFAULT_MEM_LOG_FREQ_MS                  = 0
 DEFAULT_STATS_LOG_FREQ_MS                = 15000
 DEFAULT_UNUSED_LIFETIME_MS               = 15000
 DEFAULT_WEBSOCKET_MAX_MESSAGE_SIZE_BYTES = 20*1024*1024
+DEFAULT_SESSION_TOKEN_EXPIRATION         = 300
 
 __all__ = (
     'BokehTornado',
@@ -77,7 +74,7 @@ class BokehTornado(TornadoApplication):
 
             .. code-block:: python
 
-                applications = { '/' : applications }
+                applications = {{ '/' : applications }}
 
             When a connection comes in to a given path, the associate
             Application is used to generate a new document for the session.
@@ -121,23 +118,27 @@ class BokehTornado(TornadoApplication):
             (default: True)
 
         keep_alive_milliseconds (int, optional) :
-            Number of milliseconds between keep-alive pings (default: 37000)
+            Number of milliseconds between keep-alive pings
+            (default: {DEFAULT_KEEP_ALIVE_MS})
 
             Pings normally required to keep the websocket open. Set to 0 to
             disable pings.
 
         check_unused_sessions_milliseconds (int, optional) :
             Number of milliseconds between checking for unused sessions
-            (default: 17000)
+            (default: {DEFAULT_CHECK_UNUSED_MS})
 
         unused_session_lifetime_milliseconds (int, optional) :
-            Number of milliseconds for unused session lifetime (default: 15000)
+            Number of milliseconds for unused session lifetime
+            (default: {DEFAULT_UNUSED_LIFETIME_MS})
 
         stats_log_frequency_milliseconds (int, optional) :
-            Number of milliseconds between logging stats (default: 15000)
+            Number of milliseconds between logging stats
+            (default: {DEFAULT_STATS_LOG_FREQ_MS})
 
         mem_log_frequency_milliseconds (int, optional) :
-            Number of milliseconds between logging memory information (default: 0)
+            Number of milliseconds between logging memory information
+            (default: {DEFAULT_MEM_LOG_FREQ_MS})
 
             Enabling this feature requires the optional dependency ``psutil`` to be
             installed.
@@ -161,9 +162,7 @@ class BokehTornado(TornadoApplication):
 
         websocket_max_message_size_bytes (int, optional):
             Set the Tornado ``websocket_max_message_size`` value.
-            (default: 20*1024*1024)
-
-            NOTE: This setting has effect ONLY for Tornado>=4.5
+            (default: {DEFAULT_WEBSOCKET_MAX_MESSAGE_SIZE_BYTES})
 
         index (str, optional):
             Path to a Jinja2 template to use for the root URL
@@ -171,6 +170,28 @@ class BokehTornado(TornadoApplication):
         auth_provider (AuthProvider, optional):
             An AuthProvider instance
 
+        include_headers (list, optional) :
+            List of request headers to include in session context
+            (by default all headers are included)
+
+        exclude_headers (list, optional) :
+            List of request headers to exclude in session context
+            (by default all headers are included)
+
+        include_cookies (list, optional) :
+            List of cookies to include in session context
+            (by default all cookies are included)
+
+        exclude_cookies (list, optional) :
+            List of cookies to exclude in session context
+            (by default all cookies are included)
+
+        session_token_expiration (int, optional) :
+            Duration in seconds that a new session token is valid
+            for session creation. After the expiry time has elapsed,
+            the token will not be able create a new session
+            (default: {DEFAULT_SESSION_TOKEN_EXPIRATION})
+
     Any additional keyword arguments are passed to ``tornado.web.Application``.
     '''
 
@@ -193,11 +214,32 @@ class BokehTornado(TornadoApplication):
                  index=None,
                  auth_provider=NullAuth(),
                  xsrf_cookies=False,
+                 include_headers=None,
+                 include_cookies=None,
+                 exclude_headers=None,
+                 exclude_cookies=None,
+                 session_token_expiration=DEFAULT_SESSION_TOKEN_EXPIRATION,
                  **kwargs):
 
         # This will be set when initialize is called
         self._loop = None
 
+        from bokeh.application.handlers.function import FunctionHandler
+        from bokeh.application.handlers.document_lifecycle import DocumentLifecycleHandler
+
+        if callable(applications):
+            applications = Application(FunctionHandler(applications))
+
+        if isinstance(applications, Application):
+            applications = { '/' : applications }
+
+        for k, v in list(applications.items()):
+            if callable(v):
+                applications[k] = Application(FunctionHandler(v))
+            if all(not isinstance(handler, DocumentLifecycleHandler)
+                    for handler in applications[k]._handlers):
+                applications[k].add(DocumentLifecycleHandler())
+
         if isinstance(applications, Application):
             applications = { '/' : applications }
 
@@ -228,7 +270,7 @@ class BokehTornado(TornadoApplication):
         self._check_unused_sessions_milliseconds = check_unused_sessions_milliseconds
 
         if unused_session_lifetime_milliseconds <= 0:
-            raise ValueError("check_unused_sessions_milliseconds must be > 0")
+            raise ValueError("unused_session_lifetime_milliseconds must be > 0")
         elif unused_session_lifetime_milliseconds != DEFAULT_UNUSED_LIFETIME_MS:
             log.info("Unused sessions last for %d milliseconds", unused_session_lifetime_milliseconds)
         self._unused_session_lifetime_milliseconds = unused_session_lifetime_milliseconds
@@ -269,6 +311,23 @@ class BokehTornado(TornadoApplication):
         if xsrf_cookies:
             log.info("XSRF cookie protection enabled")
 
+        if session_token_expiration <= 0:
+            raise ValueError("session_token_expiration must be > 0")
+        else:
+            self._session_token_expiration = session_token_expiration
+
+        if exclude_cookies and include_cookies:
+            raise ValueError("Declare either an include or an exclude list"
+                             "for the cookies, not both.")
+        self._exclude_cookies = exclude_cookies
+        self._include_cookies = include_cookies
+
+        if exclude_headers and include_headers:
+            raise ValueError("Declare either an include or an exclude list"
+                             "for the headers, not both.")
+        self._exclude_headers = exclude_headers
+        self._include_headers = include_headers
+
         if extra_websocket_origins is None:
             self._websocket_origins = set()
         else:
@@ -334,7 +393,7 @@ class BokehTornado(TornadoApplication):
         for line in pformat(all_patterns, width=60).split("\n"):
             log.debug("  " + line)
 
-        super(BokehTornado, self).__init__(all_patterns,
+        super().__init__(all_patterns,
                                            websocket_max_message_size=websocket_max_message_size_bytes,
                                            **kwargs)
 
@@ -367,6 +426,15 @@ class BokehTornado(TornadoApplication):
         else:
             self._ping_job = None
 
+
+
+    @property
+    def applications(self):
+        ''' The configured applications
+
+        '''
+        return self._applications
+
     @property
     def app_paths(self):
         ''' A list of all application paths for all Bokeh applications
@@ -413,6 +481,36 @@ class BokehTornado(TornadoApplication):
         '''
         return self._secret_key
 
+    @property
+    def include_cookies(self):
+        ''' A list of request cookies to make available in the session
+        context.
+
+        '''
+        return self._include_cookies
+
+    @property
+    def include_headers(self):
+        ''' A list of request headers to make available in the session
+        context.
+
+        '''
+        return self._include_headers
+
+    @property
+    def exclude_cookies(self):
+        ''' A list of request cookies to exclude in the session context.
+
+        '''
+        return self._exclude_cookies
+
+    @property
+    def exclude_headers(self):
+        ''' A list of request headers to exclude in the session context.
+
+        '''
+        return self._exclude_headers
+
     @property
     def sign_sessions(self):
         ''' Whether this Bokeh Server Tornado Application has been configured
@@ -430,6 +528,16 @@ class BokehTornado(TornadoApplication):
         '''
         return self._generate_session_ids
 
+    @property
+    def session_token_expiration(self):
+        ''' Duration in seconds that a new session token is valid for
+        session creation.
+
+        After the expiry time has elapsed, the token will not be able
+        create a new session.
+        '''
+        return self._session_token_expiration
+
     def resources(self, absolute_url=None):
         ''' Provide a :class:`~bokeh.resources.Resources` that specifies where
         Bokeh application sessions should load BokehJS resources from.
@@ -440,8 +548,11 @@ class BokehTornado(TornadoApplication):
                 relative URLs are used (default: None)
 
         '''
-        root_url = absolute_url + self._prefix if absolute_url else self._prefix
-        return Resources(mode="server", root_url=root_url, path_versioner=StaticHandler.append_version)
+        mode = settings.resources(default="server")
+        if mode == "server":
+            root_url = urljoin(absolute_url, self._prefix) if absolute_url else self._prefix
+            return Resources(mode="server", root_url=root_url, path_versioner=StaticHandler.append_version)
+        return Resources(mode=mode)
 
     def start(self):
         ''' Start the Bokeh Server application.
@@ -459,7 +570,7 @@ class BokehTornado(TornadoApplication):
             self._ping_job.start()
 
         for context in self._applications.values():
-            context.run_load_hook()
+            self._loop.spawn_callback(context.run_load_hook)
 
     def stop(self, wait=True):
         ''' Stop the Bokeh Server application.
@@ -531,12 +642,11 @@ class BokehTornado(TornadoApplication):
 
     # Periodic Callbacks ------------------------------------------------------
 
-    @gen.coroutine
-    def _cleanup_sessions(self):
+    async def _cleanup_sessions(self):
         log.trace("Running session cleanup job")
         for app in self._applications.values():
-            yield app._cleanup_sessions(self._unused_session_lifetime_milliseconds)
-        raise gen.Return(None)
+            await app._cleanup_sessions(self._unused_session_lifetime_milliseconds)
+        return None
 
     def _log_stats(self):
         log.trace("Running stats log job")
@@ -589,3 +699,22 @@ class BokehTornado(TornadoApplication):
 #-----------------------------------------------------------------------------
 # Code
 #-----------------------------------------------------------------------------
+
+BokehTornado.__doc__ = format_docstring(
+    BokehTornado.__doc__,
+    DEFAULT_CHECK_UNUSED_MS=DEFAULT_CHECK_UNUSED_MS,
+    DEFAULT_KEEP_ALIVE_MS=DEFAULT_KEEP_ALIVE_MS,
+    DEFAULT_MEM_LOG_FREQ_MS=DEFAULT_MEM_LOG_FREQ_MS,
+    DEFAULT_STATS_LOG_FREQ_MS=DEFAULT_STATS_LOG_FREQ_MS,
+    DEFAULT_UNUSED_LIFETIME_MS=DEFAULT_UNUSED_LIFETIME_MS,
+    DEFAULT_WEBSOCKET_MAX_MESSAGE_SIZE_BYTES=DEFAULT_WEBSOCKET_MAX_MESSAGE_SIZE_BYTES,
+    DEFAULT_SESSION_TOKEN_EXPIRATION=DEFAULT_SESSION_TOKEN_EXPIRATION,
+)
+
+# See https://github.com/bokeh/bokeh/issues/9507
+if sys.platform == 'win32' and sys.version_info[:3] >= (3, 8, 0):
+    import asyncio
+    if type(asyncio.get_event_loop_policy()) is asyncio.WindowsProactorEventLoopPolicy:
+        # WindowsProactorEventLoopPolicy is not compatible with tornado 6
+        # fallback to the pre-3.8 default of WindowsSelectorEventLoopPolicy
+        asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())
