@@ -1,27 +1,62 @@
+#-----------------------------------------------------------------------------
+# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
+# All rights reserved.
+#
+# The full license is in the file LICENSE.txt, distributed with this software.
+#-----------------------------------------------------------------------------
 ''' Provide a web socket handler for the Bokeh Server application.
 
 '''
-from __future__ import absolute_import, print_function
 
-import logging
+#-----------------------------------------------------------------------------
+# Boilerplate
+#-----------------------------------------------------------------------------
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
+#-----------------------------------------------------------------------------
+# Imports
+#-----------------------------------------------------------------------------
+
+# Standard library imports
+import calendar
 import codecs
+import datetime as dt
+from urllib.parse import urlparse
 
-from six.moves.urllib.parse import urlparse
+# External imports
+from tornado import locks, web
+from tornado.websocket import WebSocketClosedError, WebSocketHandler
 
-from tornado import gen, locks
-from tornado.websocket import StreamClosedError, WebSocketHandler, WebSocketClosedError
+# Bokeh imports
+from bokeh.settings import settings
+from bokeh.util.token import check_token_signature, get_session_id, get_token_payload
 
-from ..protocol_handler import ProtocolHandler
+# Bokeh imports
 from ...protocol import Protocol
 from ...protocol.exceptions import MessageError, ProtocolError, ValidationError
 from ...protocol.message import Message
 from ...protocol.receiver import Receiver
+from ..protocol_handler import ProtocolHandler
+from .auth_mixin import AuthMixin
 
-from bokeh.util.session_id import check_session_id_signature
+#-----------------------------------------------------------------------------
+# Globals and constants
+#-----------------------------------------------------------------------------
 
-class WSHandler(WebSocketHandler):
+__all__ = (
+    'WSHandler',
+)
+
+#-----------------------------------------------------------------------------
+# General API
+#-----------------------------------------------------------------------------
+
+#-----------------------------------------------------------------------------
+# Dev API
+#-----------------------------------------------------------------------------
+
+class WSHandler(AuthMixin, WebSocketHandler):
     ''' Implements a custom Tornado WebSocketHandler for the Bokeh Server.
 
     '''
@@ -34,8 +69,11 @@ class WSHandler(WebSocketHandler):
         # write_lock allows us to lock the connection to send multiple
         # messages atomically.
         self.write_lock = locks.Lock()
+
+        self._token = None
+
         # Note: tornado_app is stored as self.application
-        super(WSHandler, self).__init__(tornado_app, *args, **kw)
+        super().__init__(tornado_app, *args, **kw)
 
     def initialize(self, application_context, bokeh_websocket_path):
         pass
@@ -43,7 +81,7 @@ class WSHandler(WebSocketHandler):
     def check_origin(self, origin):
         ''' Implement a check_origin policy for Tornado to call.
 
-        The suplied origin will be compared to the Bokeh server whitelist. If the
+        The supplied origin will be compared to the Bokeh server allowlist. If the
         origin is not allow, an error will be logged and ``False`` will be returned.
 
         Args:
@@ -54,21 +92,24 @@ class WSHandler(WebSocketHandler):
             bool, True if the connection is allowed, False otherwise
 
         '''
-        from ..util import check_whitelist
+        from ..util import check_allowlist
         parsed_origin = urlparse(origin)
         origin_host = parsed_origin.netloc.lower()
 
         allowed_hosts = self.application.websocket_origins
+        if settings.allowed_ws_origin():
+            allowed_hosts = set(settings.allowed_ws_origin())
 
-        allowed = check_whitelist(origin_host, allowed_hosts)
+        allowed = check_allowlist(origin_host, allowed_hosts)
         if allowed:
             return True
         else:
             log.error("Refusing websocket connection from Origin '%s'; \
-                      use --allow-websocket-origin=%s to permit this; currently we allow origins %r",
-                      origin, origin_host, allowed_hosts)
+                      use --allow-websocket-origin=%s or set BOKEH_ALLOW_WS_ORIGIN=%s to permit this; currently we allow origins %r",
+                      origin, origin_host, origin_host, allowed_hosts)
             return False
 
+    @web.authenticated
     def open(self):
         ''' Initialize a connection to a client.
 
@@ -77,42 +118,53 @@ class WSHandler(WebSocketHandler):
 
         '''
         log.info('WebSocket connection opened')
+        token = self._token
 
-        proto_version = self.get_argument("bokeh-protocol-version", default=None)
-        if proto_version is None:
+        if self.selected_subprotocol != 'bokeh':
             self.close()
-            raise ProtocolError("No bokeh-protocol-version specified")
-
-        session_id = self.get_argument("bokeh-session-id", default=None)
-        if session_id is None:
+            raise ProtocolError("Subprotocol header is not 'bokeh'")
+        elif token is None:
             self.close()
-            raise ProtocolError("No bokeh-session-id specified")
+            raise ProtocolError("No token received in subprotocol header")
 
-        if not check_session_id_signature(session_id,
-                                          signed=self.application.sign_sessions,
-                                          secret_key=self.application.secret_key):
-            log.error("Session id had invalid signature: %r", session_id)
-            raise ProtocolError("Invalid session ID")
+        now = calendar.timegm(dt.datetime.utcnow().utctimetuple())
+        payload = get_token_payload(token)
+        if 'session_expiry' not in payload:
+            self.close()
+            raise ProtocolError("Session expiry has not been provided")
+        elif now >= payload['session_expiry']:
+            self.close()
+            raise ProtocolError("Token is expired.")
+        elif not check_token_signature(token,
+                                       signed=self.application.sign_sessions,
+                                       secret_key=self.application.secret_key):
+            session_id = get_session_id(token)
+            log.error("Token for session %r had invalid signature", session_id)
+            raise ProtocolError("Invalid token signature")
 
-        def on_fully_opened(future):
-            e = future.exception()
-            if e is not None:
-                # this isn't really an error (unless we have a
-                # bug), it just means a client disconnected
-                # immediately, most likely.
-                log.debug("Failed to fully open connection %r", e)
+        try:
+            self.application.io_loop.spawn_callback(self._async_open, self._token)
+        except Exception as e:
+            # this isn't really an error (unless we have a
+            # bug), it just means a client disconnected
+            # immediately, most likely.
+            log.debug("Failed to fully open connection %r", e)
 
-        future = self._async_open(session_id, proto_version)
-        self.application.io_loop.add_future(future, on_fully_opened)
+    def select_subprotocol(self, subprotocols):
+        log.debug('Subprotocol header received')
+        log.trace('Supplied subprotocol headers: %r', subprotocols)
+        if not len(subprotocols) == 2:
+            return None
+        self._token = subprotocols[1]
+        return subprotocols[0]
 
-    @gen.coroutine
-    def _async_open(self, session_id, proto_version):
+    async def _async_open(self, token):
         ''' Perform the specific steps needed to open a connection to a Bokeh session
 
-        Sepcifically, this method coordinates:
+        Specifically, this method coordinates:
 
         * Getting a session for a session ID (creating a new one if needed)
-        * Creating a protocol receiver and hander
+        * Creating a protocol receiver and handler
         * Opening a new ServerConnection and sending it an ACK
 
         Args:
@@ -121,18 +173,16 @@ class WSHandler(WebSocketHandler):
 
                 If no session exists with the given ID, a new session is made
 
-            proto_version (str):
-                The protocol version requested by the connecting client.
-
         Returns:
             None
 
         '''
         try:
-            yield self.application_context.create_session_if_needed(session_id, self.request)
+            session_id = get_session_id(token)
+            await self.application_context.create_session_if_needed(session_id, self.request, token)
             session = self.application_context.get_session(session_id)
 
-            protocol = Protocol(proto_version)
+            protocol = Protocol()
             self.receiver = Receiver(protocol)
             log.debug("Receiver created for %r", protocol)
 
@@ -148,12 +198,11 @@ class WSHandler(WebSocketHandler):
             raise e
 
         msg = self.connection.protocol.create('ACK')
-        yield self.send_message(msg)
+        await self.send_message(msg)
 
-        raise gen.Return(None)
+        return None
 
-    @gen.coroutine
-    def on_message(self, fragment):
+    async def on_message(self, fragment):
         ''' Process an individual wire protocol fragment.
 
         The websocket RFC specifies opcodes for distinguishing text frames
@@ -171,7 +220,7 @@ class WSHandler(WebSocketHandler):
         # report them as an unhandled Future
 
         try:
-            message = yield self._receive(fragment)
+            message = await self._receive(fragment)
         except Exception as e:
             # If you go look at self._receive, it's catching the
             # expected error types... here we have something weird.
@@ -180,14 +229,16 @@ class WSHandler(WebSocketHandler):
 
         try:
             if message:
-                work = yield self._handle(message)
+                if _message_test_port is not None:
+                    _message_test_port.received.append(message)
+                work = await self._handle(message)
                 if work:
-                    yield self._schedule(work)
+                    await self._schedule(work)
         except Exception as e:
             log.error("Handler or its work threw an exception: %r: %r", e, message, exc_info=True)
             self._internal_error("server failed to handle a message")
 
-        raise gen.Return(None)
+        return None
 
     def on_pong(self, data):
         # if we get an invalid integer or utf-8 back, either we
@@ -199,8 +250,7 @@ class WSHandler(WebSocketHandler):
         except ValueError:
             log.trace("received invalid integer in pong %r", data, exc_info=True)
 
-    @gen.coroutine
-    def send_message(self, message):
+    async def send_message(self, message):
         ''' Send a Bokeh Server protocol message to the connected client.
 
         Args:
@@ -208,23 +258,24 @@ class WSHandler(WebSocketHandler):
 
         '''
         try:
-            yield message.send(self)
-        except (WebSocketClosedError, StreamClosedError): # Tornado 4.x may raise StreamClosedError
+            if _message_test_port is not None:
+                _message_test_port.sent.append(message)
+            await message.send(self)
+        except WebSocketClosedError:
             # on_close() is / will be called anyway
-            log.warn("Failed sending message as connection was closed")
-        raise gen.Return(None)
+            log.warning("Failed sending message as connection was closed")
+        return None
 
-    @gen.coroutine
-    def write_message(self, message, binary=False, locked=True):
+    async def write_message(self, message, binary=False, locked=True):
         ''' Override parent write_message with a version that acquires a
         write lock before writing.
 
         '''
         if locked:
-            with (yield self.write_lock.acquire()):
-                yield super(WSHandler, self).write_message(message, binary)
+            with await self.write_lock.acquire():
+                await super().write_message(message, binary)
         else:
-            yield super(WSHandler, self).write_message(message, binary)
+            await super().write_message(message, binary)
 
     def on_close(self):
         ''' Clean up when the connection is closed.
@@ -234,34 +285,31 @@ class WSHandler(WebSocketHandler):
         if self.connection is not None:
             self.application.client_lost(self.connection)
 
-    @gen.coroutine
-    def _receive(self, fragment):
+    async def _receive(self, fragment):
         # Receive fragments until a complete message is assembled
         try:
-            message = yield self.receiver.consume(fragment)
-            raise gen.Return(message)
+            message = await self.receiver.consume(fragment)
+            return message
         except (MessageError, ProtocolError, ValidationError) as e:
             self._protocol_error(str(e))
-            raise gen.Return(None)
+            return None
 
-    @gen.coroutine
-    def _handle(self, message):
+    async def _handle(self, message):
         # Handle the message, possibly resulting in work to do
         try:
-            work = yield self.handler.handle(message, self.connection)
-            raise gen.Return(work)
+            work = await self.handler.handle(message, self.connection)
+            return work
         except (MessageError, ProtocolError, ValidationError) as e: # TODO (other exceptions?)
             self._internal_error(str(e))
-            raise gen.Return(None)
+            return None
 
-    @gen.coroutine
-    def _schedule(self, work):
+    async def _schedule(self, work):
         if isinstance(work, Message):
-            yield self.send_message(work)
+            await self.send_message(work)
         else:
             self._internal_error("expected a Message not " + repr(work))
 
-        raise gen.Return(None)
+        return None
 
     def _internal_error(self, message):
         log.error("Bokeh Server internal error: %s, closing connection", message)
@@ -270,3 +318,16 @@ class WSHandler(WebSocketHandler):
     def _protocol_error(self, message):
         log.error("Bokeh Server protocol error: %s, closing connection", message)
         self.close(10001, message)
+
+#-----------------------------------------------------------------------------
+# Private API
+#-----------------------------------------------------------------------------
+
+# This is an undocumented API purely for harvesting low level messages
+# for testing. When needed it will be set by the testing machinery, and
+# should not be used for any other purpose.
+_message_test_port = None
+
+#-----------------------------------------------------------------------------
+# Code
+#-----------------------------------------------------------------------------
