@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -77,10 +77,19 @@ global system configuration (not yet implemented)
     Future support is planned to load Bokeh settings from global system
     configurations.
 
-implicit defaults
-    These are default values defined by the setting declarations. There are
-    regular defaults, as well as "dev" defaults that are used when the
-    environment variable ``BOKEH_DEV=yes`` is set.
+local defaults
+    These are default values defined when accessing the setting:
+
+    .. code-block:: python
+
+        settings.resources(default="server")
+
+    Local defaults have lower precendence than every other setting mechanism
+    except global defaults.
+
+global defaults
+    These are default values defined by the setting declarations. They have
+    lower precedence than every other setting mechanism.
 
 If no value is obtained after searching all of these locations, then a
 RuntimeError will be raised.
@@ -98,9 +107,7 @@ There are a few methods on the ``settings`` object:
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
@@ -111,6 +118,7 @@ log = logging.getLogger(__name__)
 import codecs
 import os
 from os.path import abspath, expanduser, join
+from typing import Optional
 
 # External imports
 import yaml
@@ -168,7 +176,7 @@ def convert_bool(value):
     if val in ["no", "0", "off", "false", "False"]:
         return False
 
-    raise ValueError("Cannot convert {} to boolean value".format(value))
+    raise ValueError(f"Cannot convert {value} to boolean value")
 
 def convert_str_seq(value):
     ''' Convert a string to a lit of strings
@@ -188,8 +196,8 @@ def convert_str_seq(value):
 
     try:
         return value.split(",")
-    except:
-        raise ValueError("Cannot convert {} to list value".format(value))
+    except Exception:
+        raise ValueError(f"Cannot convert {value} to list value")
 
 _log_levels = {
     "CRITICAL" : logging.CRITICAL,
@@ -240,18 +248,19 @@ class _Unset: pass
 def is_dev():
     return convert_bool(os.environ.get("BOKEH_DEV", False))
 
-class PrioritizedSetting(object):
+class PrioritizedSetting:
     ''' Return a value for a global setting according to configuration precedence.
 
     The following methods are searched in order for the setting:
 
-    6. immediately supplied values
-    5. previously user-set values (e.g. set from command line)
-    4. user-specified config override file
-    3. environment variable
-    2. local user config file
-    1. global system config file (not yet implemented)
-    0. implicit defaults
+    7. immediately supplied values
+    6. previously user-set values (e.g. set from command line)
+    5. user-specified config override file
+    4. environment variable
+    3. local user config file
+    2. global system config file (not yet implemented)
+    1. local defaults
+    0. global defaults
 
     Ref: https://stackoverflow.com/a/11077282/3406693
 
@@ -278,7 +287,7 @@ class PrioritizedSetting(object):
         self._parent = None
         self._user_value = _Unset
 
-    def __call__(self, value=None):
+    def __call__(self, value=None, default=_Unset):
         '''Return the setting value according to the standard precedence.
 
         Args:
@@ -286,6 +295,10 @@ class PrioritizedSetting(object):
                 An optional immediate value. If not None, the value will
                 be converted, then returned.
 
+            default (any, optional):
+                An optional default value that only takes precendence over
+                implicit default values specified on the property itself.
+
         Returns:
             str or int or float
 
@@ -293,33 +306,39 @@ class PrioritizedSetting(object):
             RuntimeError
         '''
 
-        # 6. immediate values
+        # 7. immediate values
         if value is not None:
             return self._convert(value)
 
-        # 5. previously user-set value
+        # 6. previously user-set value
         if self._user_value is not _Unset:
             return self._convert(self._user_value)
 
-        # 4. user-named config file
+        # 5. user-named config file
         if self._parent and self._name in self._parent.config_override:
             return self._convert(self._parent.config_override[self._name])
 
-        # 3. environment variable
+        # 4. environment variable
         if self._env_var and self._env_var in os.environ:
             return self._convert(os.environ[self._env_var])
 
-        # 2. local config file
+        # 3. local config file
         if self._parent and self._name in self._parent.config_user:
             return self._convert(self._parent.config_user[self._name])
 
-        # 1. global config file
+        # 2. global config file
         if self._parent and self._name in self._parent.config_system:
             return self._convert(self._parent.config_system[self._name])
 
-        # 0. implicit defaults
+        # 1.5 (undocumented) dev defaults take precedence over other defaults
         if is_dev() and self._dev_default is not _Unset:
             return self._convert(self._dev_default)
+
+        # 1. local defaults
+        if default is not _Unset:
+            return self._convert(default)
+
+        # 0. global defaults
         if self._default is not _Unset:
             return self._convert(self._default)
 
@@ -344,7 +363,11 @@ class PrioritizedSetting(object):
         Returns:
             None
         '''
-        self._user_value = value
+        # This triggers LGTMs "mutable descriptor" warning. Since descriptors
+        # are shared among all instances, it is usually not avised to store any
+        # data directly on them. But in our case we only ever have one single
+        # instance of a Settings object.
+        self._user_value = value  # lgtm [py/mutable-descriptor]
 
     def unset_value(self):
         ''' Unset the previous user value such that the priority is reset.
@@ -385,7 +408,7 @@ _config_user_locations = (
 
 _config_system_locations = ()
 
-class Settings(object):
+class Settings:
     '''
 
     '''
@@ -437,6 +460,12 @@ class Settings(object):
     .. _webbrowser: https://docs.python.org/2/library/webbrowser.html
     """)
 
+    cdn_version = PrioritizedSetting("version", "BOKEH_CDN_VERSION", default=None, help="""
+    What version of BokehJS to use with CDN resources.
+
+    See the :class:`~bokeh.resources.Resources` class reference for full details.
+    """)
+
     cookie_secret = PrioritizedSetting("cookie_secret", "BOKEH_COOKIE_SECRET", default=None, help="""
     Configure the ``cookie_secret`` setting in Tornado. This value is required
     if you use ``get_secure_cookie`` or ``set_secure_cookie``.  It should be a
@@ -495,6 +524,10 @@ class Settings(object):
     Whether Bokeh should use minified BokehJS resources.
     """)
 
+    legacy = PrioritizedSetting("legacy", "BOKEH_LEGACY", convert=convert_bool, default=False, dev_default=False, help="""
+    Whether Bokeh should use legacy (IE and phantomjs compatible) BokehJS resources.
+    """)
+
     nodejs_path = PrioritizedSetting("nodejs_path", "BOKEH_NODEJS_PATH", default=None, help="""
     Path to the Node executable.
 
@@ -510,15 +543,6 @@ class Settings(object):
     Setting this value to False may afford a small performance improvement.
     """)
 
-    phantomjs_path = PrioritizedSetting("phantomjs_path", "BOKEH_PHANTOMJS_PATH", default=None, help="""
-    Path to the PhantomJS executable.
-
-    PhantomJS is an optional dependency that is required for PNG and SVG export.
-    Bokeh will try to automatically locate an installed PhantomJS executable.
-    Use this environment variable to override the location Bokeh finds, or to
-    point to a non-standard location.
-    """)
-
     pretty = PrioritizedSetting("pretty", "BOKEH_PRETTY", default=False, dev_default=True, help="""
     Whether JSON strings should be pretty-printed.
     """)
@@ -564,7 +588,7 @@ class Settings(object):
     Whether Bokeh should use simple integers for model IDs (starting at 1000).
 
     If False, Bokeh will use UUIDs for object identifiers. This might be needed,
-    e.g., if mulitple processes are contributing to a single Bokeh Document.
+    e.g., if multiple processes are contributing to a single Bokeh Document.
     """)
 
     ssl_certfile = PrioritizedSetting("ssl_certfile", "BOKEH_SSL_CERTFILE", default=None, help="""
@@ -583,12 +607,6 @@ class Settings(object):
     Whether validation checks should be strict (i.e. raise errors).
     """)
 
-    version = PrioritizedSetting("version", "BOKEH_VERSION", default=None, help="""
-    What version of BokehJS to use with CDN resources.
-
-    See the :class:`~bokeh.resources.Resources` class reference for full details.
-    """)
-
     xsrf_cookies = PrioritizedSetting("xsrf_cookies", "BOKEH_XSRF_COOKIES", default=False, convert=convert_bool, help="""
     Whether to enable Tornado XSRF cookie protection on the Bokeh server. This
     is only applicable when also using an auth module or custom handlers. See
@@ -639,9 +657,9 @@ class Settings(object):
         try:
             self._config_override = yaml.load(open(abspath(location)), Loader=yaml.SafeLoader)
         except Exception:
-            raise RuntimeError("Could not load Bokeh config file: {}".format(location))
+            raise RuntimeError(f"Could not load Bokeh config file: {location}")
 
-    def secret_key_bytes(self):
+    def secret_key_bytes(self) -> Optional[bytes]:
         ''' Return the secret_key, converted to bytes and cached.
 
         '''
@@ -657,7 +675,7 @@ class Settings(object):
         for location in locations:
             try:
                 return yaml.load(open(location), Loader=yaml.SafeLoader)
-            except:
+            except Exception:
                 pass
         return {}
 
