@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -44,9 +44,7 @@ in conjunction with the :ref:`bokeh.sphinxext.bokeh_autodoc` extension.
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
@@ -56,14 +54,17 @@ log = logging.getLogger(__name__)
 # Standard library imports
 import importlib
 import json
+import warnings
 
 # External imports
 from docutils.parsers.rst.directives import unchanged
-
 from sphinx.errors import SphinxError
 
 # Bokeh imports
-from ..model import MetaModel
+from bokeh.model import Model
+from bokeh.util.warnings import BokehDeprecationWarning
+
+# Bokeh imports
 from .bokeh_directive import BokehDirective, py_sig_re
 from .templates import MODEL_DETAIL
 
@@ -112,10 +113,15 @@ class BokehModelDirective(BokehDirective):
         if model is None:
             raise SphinxError("Unable to generate reference docs for %s, no model '%s' in %s" % (model_name, model_name, module_name))
 
-        if type(model) != MetaModel:
-            raise SphinxError("Unable to generate reference docs for %s, model '%s' is not of type MetaModel" % (model_name, model_name))
+        if not issubclass(model, Model):
+            raise SphinxError("Unable to generate reference docs for %s, model '%s' is a subclass of Model" % (model_name, model_name))
 
-        model_obj = model()
+        # We may need to instantiate deprecated objects as part of documenting
+        # them in the reference guide. Suppress any warnings here to keep the
+        # docs build clean just for this case
+        with warnings.catch_warnings():
+            warnings.filterwarnings("ignore", category=BokehDeprecationWarning)
+            model_obj = model()
 
         model_json = json.dumps(
             model_obj.to_json(include_defaults=True),
