@@ -1,18 +1,19 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
 #-----------------------------------------------------------------------------
-""" Thoroughly document Bokeh property attributes.
+''' Thoroughly document Bokeh property attributes.
 
-The ``bokeh-prop`` directive generates useful type information
-for the property attribute, including cross links to the relevant
-property types. Additionally, any per-attribute docstrings are
-also displayed.
+The ``bokeh-prop`` directive generates documentation for Bokeh model properties,
+including cross links to the relevant property types. Additionally, any
+per-attribute help strings are also displayed.
 
-This directive takes the path to an attribute on a Bokeh
-model class as an argument::
+This directive takes the name *(class.attr)* of a Bokeh property as its
+argument and the module as an option:
+
+.. code-block:: rest
 
     .. bokeh-prop:: Bar.thing
         :module: bokeh.sphinxext.sample
@@ -20,10 +21,12 @@ model class as an argument::
 Examples
 --------
 
-For the following definition of ``bokeh.sphinxext.sample.Bar``::
+For the following definition of ``bokeh.sphinxext.sample.Bar``:
+
+.. code-block:: python
 
     class Bar(Model):
-        ''' This is a Bar model. '''
+        """ This is a Bar model. """
         thing = List(Int, help="doc for thing")
 
 the above usage yields the output:
@@ -31,14 +34,16 @@ the above usage yields the output:
     .. bokeh-prop:: Bar.thing
         :module: bokeh.sphinxext.sample
 
-"""
+
+The ``bokeh-prop`` direction may be used explicitly, but it can also be used
+in conjunction with the :ref:`bokeh.sphinxext.bokeh_autodoc` extension.
+
+'''
 
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
@@ -48,12 +53,15 @@ log = logging.getLogger(__name__)
 # Standard library imports
 import importlib
 import textwrap
+import warnings
 
 # External imports
 from docutils.parsers.rst.directives import unchanged
-
 from sphinx.errors import SphinxError
 
+# Bokeh imports
+from bokeh.util.warnings import BokehDeprecationWarning
+
 # Bokeh imports
 from .bokeh_directive import BokehDirective
 from .templates import PROP_DETAIL
@@ -79,8 +87,7 @@ class BokehPropDirective(BokehDirective):
 
     has_content = True
     required_arguments = 1
-    optional_arguments = 2
-
+    optional_arguments = 1
     option_spec = {
         'module': unchanged
     }
@@ -98,7 +105,12 @@ class BokehPropDirective(BokehDirective):
         if model is None:
             raise SphinxError("Unable to generate reference docs for %s, no model '%s' in %s" % (self.arguments[0], model_name, self.options['module']))
 
-        model_obj = model()
+        # We may need to instantiate deprecated objects as part of documenting
+        # them in the reference guide. Suppress any warnings here to keep the
+        # docs build clean just for this case
+        with warnings.catch_warnings():
+            warnings.filterwarnings("ignore", category=BokehDeprecationWarning)
+            model_obj = model()
 
         try:
             descriptor = getattr(model_obj.__class__, prop_name)
@@ -115,6 +127,7 @@ class BokehPropDirective(BokehDirective):
         return self._parse(rst_text, "<bokeh-prop>")
 
 def setup(app):
+    ''' Required Sphinx extension setup function. '''
     app.add_directive_to_domain('py', 'bokeh-prop', BokehPropDirective)
 
 #-----------------------------------------------------------------------------
