@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -12,23 +12,23 @@ transformations to data fields or ``ColumnDataSource`` expressions.
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
 # Imports
 #-----------------------------------------------------------------------------
 
-# Standard library imports
-
-# External imports
-
 # Bokeh imports
 from .core.properties import expr, field
 from .models.expressions import CumSum, Stack
-from .models.mappers import CategoricalColorMapper, CategoricalMarkerMapper, CategoricalPatternMapper, LinearColorMapper, LogColorMapper
+from .models.mappers import (
+    CategoricalColorMapper,
+    CategoricalMarkerMapper,
+    CategoricalPatternMapper,
+    LinearColorMapper,
+    LogColorMapper,
+)
 from .models.transforms import Dodge, Jitter
 
 #-----------------------------------------------------------------------------
@@ -53,7 +53,7 @@ __all__ = (
 #-----------------------------------------------------------------------------
 
 def cumsum(field, include_zero=False):
-    ''' Create a Create a ``DataSpec`` dict to generate a ``CumSum`` expression
+    ''' Create a ``DataSpec`` dict to generate a ``CumSum`` expression
     for a ``ColumnDataSource``.
 
     Examples:
@@ -66,7 +66,7 @@ def cumsum(field, include_zero=False):
 
         will generate a ``CumSum`` expressions that sum the ``"angle"`` column
         of a data source. For the ``start_angle`` value, the cumulative sums
-        will start with a zero value. For ``start_angle``, no initial zero will
+        will start with a zero value. For ``end_angle``, no initial zero will
         be added (i.e. the sums will start with the first angle value, and
         include the last).
 
@@ -74,7 +74,7 @@ def cumsum(field, include_zero=False):
     return expr(CumSum(field=field, include_zero=include_zero))
 
 def dodge(field_name, value, range=None):
-    ''' Create a ``DataSpec`` dict that applies a client-side ``Jitter``
+    ''' Create a ``DataSpec`` dict that applies a client-side ``Dodge``
     transformation to a ``ColumnDataSource`` column.
 
     Args:
