@@ -1,11 +1,10 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2017, Anaconda, Inc. All rights reserved.
-#
-# Powered by the Bokeh Development Team.
+# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
+# All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
 #-----------------------------------------------------------------------------
-''' Functions useful for dealing with hexacognal tilings.
+''' Functions useful for dealing with hexagonal tilings.
 
 For more information on the concepts employed here, see this informative page
 
@@ -16,9 +15,7 @@ For more information on the concepts employed here, see this informative page
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
@@ -26,27 +23,34 @@ log = logging.getLogger(__name__)
 #-----------------------------------------------------------------------------
 
 # Standard library imports
+from typing import Any, Tuple
 
 # External imports
 import numpy as np
 
 # Bokeh imports
-from .dependencies import import_optional
+from .dependencies import import_required
 
 #-----------------------------------------------------------------------------
 # Globals and constants
 #-----------------------------------------------------------------------------
 
+__all__ = (
+    'axial_to_cartesian',
+    'cartesian_to_axial',
+    'hexbin',
+)
+
 #-----------------------------------------------------------------------------
 # General API
 #-----------------------------------------------------------------------------
 
-def axial_to_cartesian(q, r, size, orientation, aspect_scale=1):
+def axial_to_cartesian(q: Any, r: Any, size: float, orientation: str, aspect_scale: float = 1) -> Tuple[Any, Any]:
     ''' Map axial *(q,r)* coordinates to cartesian *(x,y)* coordinates of
     tiles centers.
 
     This function can be useful for positioning other Bokeh glyphs with
-    cartesian coordinates in relationto a hex tiling.
+    cartesian coordinates in relation to a hex tiling.
 
     This function was adapted from:
 
@@ -92,7 +96,7 @@ def axial_to_cartesian(q, r, size, orientation, aspect_scale=1):
 
     return (x, y)
 
-def cartesian_to_axial(x, y, size, orientation, aspect_scale=1):
+def cartesian_to_axial(x: Any, y: Any, size: float, orientation: str, aspect_scale: float = 1) -> Tuple[Any, Any]:
     ''' Map Cartesion *(x,y)* points to axial *(q,r)* coordinates of enclosing
     tiles.
 
@@ -144,11 +148,11 @@ def cartesian_to_axial(x, y, size, orientation, aspect_scale=1):
 
     return _round_hex(q, r)
 
-def hexbin(x, y, size, orientation="pointytop", aspect_scale=1):
+def hexbin(x: Any, y: Any, size: float, orientation: str = "pointytop", aspect_scale: float = 1) -> Any:
     ''' Perform an equal-weight binning of data points into hexagonal tiles.
 
-    For more sophiscticated use cases, e.g. weighted binning or scaling
-    individual tiles proprtional to some other quantity, consider using
+    For more sophisticated use cases, e.g. weighted binning or scaling
+    individual tiles proportional to some other quantity, consider using
     HoloViews.
 
     Args:
@@ -174,7 +178,7 @@ def hexbin(x, y, size, orientation="pointytop", aspect_scale=1):
 
             When working with a plot with ``aspect_scale != 1``, this
             parameter can be set to match the plot, in order to draw
-            regular hexagons (insted of "stretched" ones).
+            regular hexagons (instead of "stretched" ones).
 
             This is roughly equivalent to binning in "screen space", and
             it may be better to use axis-aligned rectangular bins when
@@ -191,11 +195,12 @@ def hexbin(x, y, size, orientation="pointytop", aspect_scale=1):
         Hex binning only functions on linear scales, i.e. not on log plots.
 
     '''
-    pd = import_optional('pandas')
+    pd: Any = import_required('pandas','hexbin requires pandas to be installed')
 
     q, r = cartesian_to_axial(x, y, size, orientation, aspect_scale=aspect_scale)
 
     df = pd.DataFrame(dict(r=r, q=q))
+
     return df.groupby(['q', 'r']).size().reset_index(name='counts')
 
 #-----------------------------------------------------------------------------
@@ -206,7 +211,7 @@ def hexbin(x, y, size, orientation="pointytop", aspect_scale=1):
 # Private API
 #-----------------------------------------------------------------------------
 
-def _round_hex(q, r):
+def _round_hex(q: Any, r: Any) -> Tuple[Any, Any]:
     ''' Round floating point axial hex coordinates to integer *(q,r)*
     coordinates.
 
