@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -18,9 +18,7 @@ performance and efficiency. The list of supported dtypes is:
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
@@ -30,18 +28,18 @@ log = logging.getLogger(__name__)
 # Standard library imports
 import base64
 import datetime as dt
-import math
 import sys
-from threading import Lock
 import uuid
+from math import isinf, isnan
+from threading import Lock
 
 # External imports
 import numpy as np
 
 # Bokeh imports
 from ..settings import settings
-from .string import format_docstring
 from .dependencies import import_optional
+from .string import format_docstring
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -49,7 +47,7 @@ from .dependencies import import_optional
 
 pd = import_optional('pandas')
 
-BINARY_ARRAY_TYPES = set([
+BINARY_ARRAY_TYPES = {
     np.dtype(np.float32),
     np.dtype(np.float64),
     np.dtype(np.uint8),
@@ -58,14 +56,13 @@ BINARY_ARRAY_TYPES = set([
     np.dtype(np.int16),
     np.dtype(np.uint32),
     np.dtype(np.int32),
-])
+}
 
-DATETIME_TYPES = set([
-    dt.datetime,
-    dt.date,
+DATETIME_TYPES = {
     dt.time,
+    dt.datetime,
     np.datetime64,
-])
+}
 
 if pd:
     try:
@@ -86,6 +83,7 @@ __doc__ = format_docstring(__doc__, binary_array_types="\n".join("* ``np." + str
 
 __all__ = (
     'array_encoding_disabled',
+    'convert_date_to_datetime',
     'convert_datetime_array',
     'convert_datetime_type',
     'convert_timedelta_type',
@@ -133,6 +131,18 @@ def is_timedelta_type(obj):
     '''
     return isinstance(obj, (dt.timedelta, np.timedelta64))
 
+def convert_date_to_datetime(obj):
+    ''' Convert a date object to a datetime
+
+    Args:
+        obj (date) : the object to convert
+
+    Returns:
+        datetime
+
+    '''
+    return (dt.datetime(*obj.timetuple()[:6]) - DT_EPOCH).total_seconds() * 1000
+
 def convert_timedelta_type(obj):
     ''' Convert any recognized timedelta value to floating point absolute
     milliseconds.
@@ -177,11 +187,12 @@ def convert_datetime_type(obj):
     # Datetime (datetime is a subclass of date)
     elif isinstance(obj, dt.datetime):
         diff = obj.replace(tzinfo=None) - DT_EPOCH
-        return diff.total_seconds() * 1000.
+        return diff.total_seconds() * 1000
 
+    # XXX (bev) ideally this would not be here "dates are not datetimes"
     # Date
     elif isinstance(obj, dt.date):
-        return (dt.datetime(*obj.timetuple()[:6]) - DT_EPOCH).total_seconds() * 1000
+        return convert_date_to_datetime(obj)
 
     # NumPy datetime64
     elif isinstance(obj, np.datetime64):
@@ -209,32 +220,20 @@ def convert_datetime_array(array):
     if not isinstance(array, np.ndarray):
         return array
 
-    try:
-        dt2001 = np.datetime64('2001')
-        legacy_datetime64 = (dt2001.astype('int64') ==
-                             dt2001.astype('datetime64[ms]').astype('int64'))
-    except AttributeError as e:
-        if e.args == ("'module' object has no attribute 'datetime64'",):
-            # for compatibility with PyPy that doesn't have datetime64
-            if 'PyPy' in sys.version:
-                legacy_datetime64 = False
-                pass
-            else:
-                raise e
-        else:
-            raise e
-
     # not quite correct, truncates to ms..
     if array.dtype.kind == 'M':
-        if legacy_datetime64:
-            if array.dtype == np.dtype('datetime64[ns]'):
-                array = array.astype('int64') / 10**6.0
-        else:
-            array =  array.astype('datetime64[us]').astype('int64') / 1000.
+        array =  array.astype('datetime64[us]').astype('int64') / 1000.
 
     elif array.dtype.kind == 'm':
         array = array.astype('timedelta64[us]').astype('int64') / 1000.
 
+    # XXX (bev) special case dates, not great
+    elif array.dtype.kind == 'O' and len(array) > 0 and isinstance(array[0], dt.date):
+        try:
+            array = array.astype('datetime64[us]').astype('int64') / 1000.
+        except Exception:
+            pass
+
     return array
 
 def make_id():
@@ -420,30 +419,26 @@ def serialize_array(array, force_list=False, buffers=None):
     else:
         return encode_binary_dict(array, buffers)
 
-def traverse_data(obj, use_numpy=True, buffers=None):
+def traverse_data(obj, buffers=None):
     ''' Recursively traverse an object until a flat list is found.
 
-    If NumPy is available, the flat list is converted to a numpy array
-    and passed to transform_array() to handle ``nan``, ``inf``, and
-    ``-inf``.
-
-    Otherwise, iterate through all items, converting non-JSON items
+    The flat list is converted to a numpy array and passed to transform_array()
+    to handle ``nan``, ``inf``, and ``-inf``.
 
     Args:
         obj (list) : a list of values or lists
-        use_numpy (bool, optional) toggle NumPy as a dependency for testing
-            This argument is only useful for testing (default: True)
+
     '''
-    if use_numpy and all(isinstance(el, np.ndarray) for el in obj):
+    if all(isinstance(el, np.ndarray) for el in obj):
         return [transform_array(el, buffers=buffers) for el in obj]
     obj_copy = []
     for item in obj:
         # Check the base/common case first for performance reasons
         # Also use type(x) is float because it's faster than isinstance
         if type(item) is float:
-            if math.isnan(item):
+            if isnan(item):
                 item = 'NaN'
-            elif math.isinf(item):
+            elif isinf(item):
                 if item > 0:
                     item = 'Infinity'
                 else:
@@ -526,7 +521,7 @@ def encode_binary_dict(array, buffers):
         '__buffer__'  : buffer_id,
         'shape'       : array.shape,
         'dtype'       : array.dtype.name,
-        'order'       : sys.byteorder
+        'order'       : sys.byteorder,
     }
 
 def encode_base64_dict(array):
@@ -551,9 +546,10 @@ def encode_base64_dict(array):
 
     '''
     return {
-        '__ndarray__'  : base64.b64encode(array.data).decode('utf-8'),
-        'shape'        : array.shape,
-        'dtype'        : array.dtype.name
+        '__ndarray__' : base64.b64encode(array.data).decode('utf-8'),
+        'shape'       : array.shape,
+        'dtype'       : array.dtype.name,
+        'order'       : sys.byteorder,
     }
 
 def decode_base64_dict(data):
